# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import os
import sys
import time

import cv2
from PySide6.QtCore import Qt, QThread, Signal, Slot
from PySide6.QtGui import QAction, QImage, QKeySequence, QPixmap
from PySide6.QtWidgets import (QApplication, QComboBox, QGroupBox,
                               QHBoxLayout, QLabel, QMainWindow, QPushButton,
                               QSizePolicy, QVBoxLayout, QWidget)


"""This example uses the video from a  webcam to apply pattern
detection from the OpenCV module. e.g.: face, eyes, body, etc."""


class Thread(QThread):
    updateFrame = Signal(QImage)

    def __init__(self, parent=None):
        QThread.__init__(self, parent)
        self.trained_file = None
        self.status = True
        self.cap = True

    def set_file(self, fname):
        # The data comes with the 'opencv-python' module
        self.trained_file = os.path.join(cv2.data.haarcascades, fname)

    def run(self):
        self.cap = cv2.VideoCapture(0)
        while self.status:
            cascade = cv2.CascadeClassifier(self.trained_file)
            ret, frame = self.cap.read()
            if not ret:
                continue

            # Reading frame in gray scale to process the pattern
            gray_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

            detections = cascade.detectMultiScale(gray_frame, scaleFactor=1.1,
                                                  minNeighbors=5, minSize=(30, 30))

            # Drawing green rectangle around the pattern
            for (x, y, w, h) in detections:
                pos_ori = (x, y)
                pos_end = (x + w, y + h)
                color = (0, 255, 0)
                cv2.rectangle(frame, pos_ori, pos_end, color, 2)

            # Reading the image in RGB to display it
            color_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

            # Creating and scaling QImage
            h, w, ch = color_frame.shape
            img = QImage(color_frame.data, w, h, ch * w, QImage.Format.Format_RGB888)
            scaled_img = img.scaled(640, 480, Qt.AspectRatioMode.KeepAspectRatio)

            # Emit signal
            self.updateFrame.emit(scaled_img)
        sys.exit(-1)


class Window(QMainWindow):
    def __init__(self):
        super().__init__()
        # Title and dimensions
        self.setWindowTitle("Patterns detection")
        self.setGeometry(0, 0, 800, 500)

        # Main menu bar
        self.menu = self.menuBar()
        self.menu_file = self.menu.addMenu("File")
        exit = QAction("Exit", self, triggered=qApp.quit)  # noqa: F821
        self.menu_file.addAction(exit)

        self.menu_about = self.menu.addMenu("&About")
        about = QAction("About Qt", self,
                        shortcut=QKeySequence(QKeySequence.StandardKey.HelpContents),
                        triggered=qApp.aboutQt)  # noqa: F821
        self.menu_about.addAction(about)

        # Create a label for the display camera
        self.label = QLabel(self)
        self.label.setFixedSize(640, 480)

        # Thread in charge of updating the image
        self.th = Thread(self)
        self.th.finished.connect(self.close)
        self.th.updateFrame.connect(self.setImage)

        # Model group
        self.group_model = QGroupBox("Trained model")
        self.group_model.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        model_layout = QHBoxLayout()

        self.combobox = QComboBox()
        for xml_file in os.listdir(cv2.data.haarcascades):
            if xml_file.endswith(".xml"):
                self.combobox.addItem(xml_file)

        model_layout.addWidget(QLabel("File:"), 10)
        model_layout.addWidget(self.combobox, 90)
        self.group_model.setLayout(model_layout)

        # Buttons layout
        buttons_layout = QHBoxLayout()
        self.button1 = QPushButton("Start")
        self.button2 = QPushButton("Stop/Close")
        self.button1.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        self.button2.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        buttons_layout.addWidget(self.button2)
        buttons_layout.addWidget(self.button1)

        right_layout = QHBoxLayout()
        right_layout.addWidget(self.group_model, 1)
        right_layout.addLayout(buttons_layout, 1)

        # Main layout
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        layout.addLayout(right_layout)

        # Central widget
        widget = QWidget(self)
        widget.setLayout(layout)
        self.setCentralWidget(widget)

        # Connections
        self.button1.clicked.connect(self.start)
        self.button2.clicked.connect(self.kill_thread)
        self.button2.setEnabled(False)
        self.combobox.currentTextChanged.connect(self.set_model)

    @Slot()
    def set_model(self, text):
        self.th.set_file(text)

    @Slot()
    def kill_thread(self):
        print("Finishing...")
        self.button2.setEnabled(False)
        self.button1.setEnabled(True)
        self.th.cap.release()
        cv2.destroyAllWindows()
        self.status = False
        self.th.terminate()
        # Give time for the thread to finish
        time.sleep(1)

    @Slot()
    def start(self):
        print("Starting...")
        self.button2.setEnabled(True)
        self.button1.setEnabled(False)
        self.th.set_file(self.combobox.currentText())
        self.th.start()

    @Slot(QImage)
    def setImage(self, image):
        self.label.setPixmap(QPixmap.fromImage(image))


if __name__ == "__main__":
    app = QApplication()
    w = Window()
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for connecting signals between threads'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QThread, QTimer, QObject, Signal, Slot, QCoreApplication


class Source(QObject):
    source = Signal()

    def __init__(self, *args):
        super().__init__(*args)

    @Slot()
    def emit_sig(self):
        self.source.emit()


class Target(QObject):
    def __init__(self, *args):
        super().__init__(*args)
        self.called = False

    @Slot()
    def myslot(self):
        self.called = True


class ThreadJustConnects(QThread):
    def __init__(self, source, *args):
        super().__init__(*args)
        self.source = source
        self.target = Target()

    def run(self):
        self.source.source.connect(self.target.myslot)
        self.source.source.connect(self.quit)
        self.exec()


class BasicConnection(unittest.TestCase):

    def testEmitOutsideThread(self):
        global thread_run

        app = QCoreApplication([])
        source = Source()
        thread = ThreadJustConnects(source)

        thread.finished.connect(QCoreApplication.quit)
        thread.start()

        QTimer.singleShot(50, source.emit_sig)
        app.exec()
        thread.wait()

        self.assertTrue(thread.target.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests for calling methods further than the direct parent'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QPushButton, QWidget

from helper.usesqapplication import UsesQApplication


class Dummy(QPushButton):

    def show(self):
        QWidget.show(self)
        self.called = True


class GrandParentMethod(UsesQApplication):
    def testMethod(self):
        obj = Dummy()
        obj.show()
        self.assertTrue(obj.called)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication
from PySide6.QtUiTools import QUiLoader


class TestBug913 (unittest.TestCase):

    def testIt(self):
        app = QApplication([])  # noqa: F841

        loader = QUiLoader()
        file = Path(__file__).resolve().parent / 'bug_913.ui'
        self.assertTrue(file.is_file())
        widget = loader.load(file)
        widget.tabWidget.currentIndex()  # direct child is available as member
        widget.le_first.setText('foo')  # child of QTabWidget must also be available!


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import QStandardPaths
from PySide6.QtWidgets import QWidget, QFrame, QPushButton
from PySide6.QtUiTools import loadUiType


class loadUiTypeTester(UsesQApplication):

    @unittest.skipUnless(bool(QStandardPaths.findExecutable("pyside6-uic")), "pyside6-uic missing")
    def testFunction(self):
        filePath = os.path.join(os.path.dirname(__file__), "minimal.ui")
        loaded = loadUiType(filePath)
        self.assertNotEqual(loaded, None)

        # (<class '__main__.Ui_Form'>, <class 'PySide6.QtWidgets.QFrame'>)
        generated, base = loaded

        # Generated class contains retranslateUi method
        self.assertTrue("retranslateUi" in dir(generated))

        # Base class instance will be QFrame for this example
        self.assertTrue(isinstance(base(), QFrame))

        anotherFileName = os.path.join(os.path.dirname(__file__), "test.ui")
        another = loadUiType(anotherFileName)
        self.assertNotEqual(another, None)

        generated, base = another
        # Base class instance will be QWidget for this example
        self.assertTrue(isinstance(base(), QWidget))

        w = base()
        ui = generated()
        ui.setupUi(w)

        self.assertTrue(isinstance(ui.child_object, QFrame))
        self.assertTrue(isinstance(ui.grandson_object, QPushButton))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtWidgets import QMainWindow, QMenu


class MainWindow(QMainWindow):
    def __init__(self, *args):
        self._menu = QMenu(self.dontexist)  # attribute called with invalid C++ object


class MainWindow2(QMainWindow):
    def __init__(self):
        self.show()


class Bug696(UsesQApplication):
    def testContructorInitialization(self):
        self.assertRaises(AttributeError, MainWindow)

    def testContructorInitializationAndCPPFunction(self):
        self.assertRaises(RuntimeError, MainWindow2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for methods that receive references to objects.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Reference, Str


class ExtendedReference(Reference):
    def __init__(self):
        Reference.__init__(self)
        self.uses_reference_virtual_called = False
        self.uses_const_reference_virtual_called = False
        self.reference_inc = 1
        self.const_reference_inc = 2
        self.multiplier = 333

    def usesReferenceVirtual(self, ref, inc):
        self.uses_reference_virtual_called = True
        return ref.objId() + inc + self.reference_inc

    def usesConstReferenceVirtual(self, ref, inc):
        self.uses_const_reference_virtual_called = True
        return ref.objId() + inc + self.const_reference_inc

    def alterReferenceIdVirtual(self, ref):
        ref.setObjId(ref.objId() * self.multiplier)


class ReferenceTest(unittest.TestCase):
    '''Test case for methods that receive references to objects.'''

    def testMethodThatReceivesReference(self):
        '''Test a method that receives a reference to an object as argument.'''
        objId = 123
        r = Reference(objId)
        self.assertEqual(Reference.usesReference(r), objId)

    def testCantSegFaultWhenReceiveNone(self):
        '''do not segfault when receiving None as argument.'''
        s = Str()
        self.assertFalse(bool(s))

    def testMethodThatReceivesConstReference(self):
        '''Test a method that receives a const reference to an object as argument.'''
        objId = 123
        r = Reference(objId)
        self.assertEqual(Reference.usesConstReference(r), objId)

    def testModificationOfReference(self):
        '''Tests if the identity of a reference argument is preserved when passing
           it to be altered in C++.'''
        objId = 123
        r1 = Reference(objId)
        r1.alterReferenceIdVirtual(r1)
        self.assertEqual(r1.objId(), objId * Reference.multiplier())

    def testModificationOfReferenceCallingAVirtualIndirectly(self):
        '''Tests if the identity of a reference argument is preserved when passing it
           to be altered in C++ through a method that calls a virtual method.'''
        objId = 123
        r1 = Reference(objId)
        r1.callAlterReferenceIdVirtual(r1)
        self.assertEqual(r1.objId(), objId * Reference.multiplier())

    def testModificationOfReferenceCallingAReimplementedVirtualIndirectly(self):
        '''Test if a Python override of a virtual method with a reference parameter
           called from C++ alters the argument properly.'''
        objId = 123
        r = Reference(objId)
        er = ExtendedReference()
        result = er.callAlterReferenceIdVirtual(r)  # noqa: F841
        self.assertEqual(r.objId(), objId * er.multiplier)

    def testReimplementedVirtualMethodCallWithReferenceParameter(self):
        '''Test if a Python override of a virtual method with a reference parameter
           is correctly called from C++.'''
        inc = 9
        objId = 123
        r = Reference(objId)
        er = ExtendedReference()
        result = er.callUsesReferenceVirtual(r, inc)
        self.assertEqual(result, objId + inc + er.reference_inc)

    def testReimplementedVirtualMethodCallWithConstReferenceParameter(self):
        '''Test if a Python override of a virtual method with a const reference
           parameter is correctly called from C++.'''
        inc = 9
        objId = 123
        r = Reference(objId)
        er = ExtendedReference()
        result = er.callUsesConstReferenceVirtual(r, inc)
        self.assertEqual(result, objId + inc + er.const_reference_inc)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUrl
from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import qmlRegisterType
from PySide6.QtQuick import QQuickView, QQuickItem


def register_qml_types():
    class TestClass(QQuickItem):
        def __init__(self, parent=None):
            QQuickItem.__init__(self, parent)

    qmlRegisterType(TestClass, "UserTypes", 1, 0, "TestClass")


def main():
    app = QGuiApplication([])  # noqa

    # reg qml types here
    register_qml_types()

    # force gc to run
    gc.collect()

    view = QQuickView()
    url = QUrl(__file__.replace(".py", ".qml"))
    view.setSource(url)


if __name__ == "__main__":
    main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QMainWindow, QTabWidget, QTextEdit, QSplitter
from helper.usesqapplication import UsesQApplication


class TabWidgetClear(QMainWindow):
    def __init__(self):
        super().__init__()
        self.tabWidget = QTabWidget(self)
        self.setCentralWidget(self.tabWidget)
        self.editBox = QTextEdit(self)
        self.tabWidget.addTab(self.getSplitter(), 'Test')

    def getSplitter(self):
        splitter = QSplitter()
        splitter.addWidget(self.editBox)
        return splitter

    def toggle(self):
        self.tabWidget.clear()
        self.getSplitter()


class TestTabWidgetClear(UsesQApplication):

    def testClear(self):
        self.window = TabWidgetClear()
        self.window.show()
        try:
            self.window.toggle()
        except RuntimeError as e:
            # This should never happened, PYSIDE-213
            raise e


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''QtTest QSignalSpy'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import SIGNAL
from PySide6.QtGui import QStandardItem, QStandardItemModel
from PySide6.QtTest import QSignalSpy

from helper.usesqapplication import UsesQApplication


class QSignalSpyTest(UsesQApplication):

    def setUp(self):
        super().setUp()
        self._model = QStandardItemModel(0, 1)
        self._model.appendRow(QStandardItem('text1'))

    def testStringBased(self):
        s = SIGNAL('dataChanged(QModelIndex,QModelIndex,QList<int>)')
        spy = QSignalSpy(self._model, s)
        self._model.item(0, 0).setText('text2')
        self.assertEqual(spy.count(), 1)

    def testSignal(self):
        spy = QSignalSpy(self._model.dataChanged)
        self._model.item(0, 0).setText('text3')
        self.assertEqual(spy.count(), 1)


if __name__ == '__main__':
    unittest.main()



# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.6.1
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x03\xd6\
\x00\
\x00\x0b\xa0x\x9c\xd5VMo\xdbF\x10]}8\x8e\
\xe94i\x9a\xaf&q\xddM\xdd&\x94\xe3\xa8\xb4\xe0\
\x04Ee\xb9-\x92\x221\x10\xc0n\x5c\x18\x05\x04\x83\
P$\xca! \x93\x86H\x0a*\x02\x03\x05r\xcf\xb1\
\xa7\xde\xf2\x0f\xdaC\xff@\x81\xfe\xa5^\x8avf\xf7\
Q\xbb\xfapN\xb9d\x01\x8a\x9c\xb73\xf3f\xe6-\
i\x0b!\x16\x84Y\x05\xba\xce\x8a\x81\xf0EF\xbfB\
\x14\x19\xfc\x0fK\xbc\xc3\xc5D\xabt\x9d\xb7l^e\
\xdc\xcf\xe0\xde1!\xa5\xdfJ\xc5\x95yvu\xa8\xc6\
\xbb@/R\x8d\x1c\xbcH!|\x7f\xfct\xefi5\
I;\xd5\x8d\xfb\x9eP\x04\xa5\x11\xc1\x05\x8a\xcbI\x8e\
Za\xc4\xcf\xdc\xfd9\xba>\xe1X\xdaa{\x19\xb9\
\xd9>\xa7r\xebb\x9f\x889q\x13{\x8b\xb8\xe7X\
\x01X\xc1\xc2\x8a\xc0J*Wi\x84]\xa2_\xbe\xdf\
\x22\xac\x04\xec*=q\xd3\x92\xaek\xc4;\x8f!\x14\
\x15VV\x95\xb3/\xe3u\xd8\x0b\xc0\xae\x90\xed \xb6\
\x00\xff\xbc\x16\x07\xfe\x8b\xe8\x93\xb1\xbb\xf0??a\x7f\
\x08>\xe6\xbf\x06\xfe2\xf2\x7f\x8c\xfc\x02\xfel_G\
\xfc\x12\xd97\xe0\x7f]\xcdO\xf7\x9f\xe7\xb8\x81\x9a\x96\
P\xefM\xd4\xb4\x84\xb9\x97\xac\x1a\x96-\x8e3\x86\xf3\
\xdb\xdc\xfeL\x99\xbf|\x93\xdb_\xc0\x9f\xf3W\x90\x8b\
y\x1fP\x0dEa\x0e\x14\xe3\xff\x102G\xf7\x06\xb8\
>\xc0L\xfe\x22\x94\xed\x0b\xc0x.\x7f\x03\xbb\x08\x9c\
g\x13\x83\xf3#\xe0y\x9eK\xc8\x93\xc7\x5c\x06f\xc7\
\x5c\x01\xbeK>\xf3Jo\x9d\x87\xf1-\xaan\x01X\
\x03\xda\x7f\x0a}_\xc3\xff\x160i\xe5X\xc1<\xf8\
z\x05\xecs\xf8\xf2\xde\x0f\x841\xf7m\xe0\xc2\xc2\xee\
\x00+\xa8|\xf3jf.|\xef`\xae\x9c\xf7;\xf2\
\xe7\xb9\xaeB\xabe\xd4\xbb\x0a\xff\x7f)\xc3W\xfa\xf5\
-\xe3\x9db\xa3#\xcc;\xfd\xe7\xca \xe8'a\x1c\
\xc9u\xcfs\x9cA\xab\xffs\x18\x1d\xcaA\xd0\xae\xc9\
\x81\x9f\x0d\xea\x84\xc5aGr\xb8[q^:\x92\x16\
o\xc8\x86rr\xbb\xbd\xb8\x95\xba\xeea\xcf\xdf\x0f\xfa\
i0\xdc~$77\xe5zE\xde\x96\xb5\xca\x9a\xd4\
\xdb\xf6.\xe3\x95\xba\xcaC\xf0n\x9c\x84)\xd3\xabt\
\x1b\xae\xabr\xaf\xcaZ\xd5\xab\xc8{\x9ab\x9d\x9e\xd7\
\xa4W\xf5\xd6$?\xd6\x9d\x13\xc7\x99hi(\xcc\xb2\
Z\xaa\xbd\xc7-\xfdj\xb5\xf4\x87i\xe9>\xb5\x14g\
\xe9{\xd5\x0e\xbf\xeb5\xd3M\xf1M\x92\xb6\xd2\xb0\xad\
\xa97l\x92\xba\x83\xad0J\xa5\xa9)\xea\x04\xc3\xd1\
\x96\x8a\x1a\xf5\x9e\xa4\xfd\xac\x9d\xca\xbd\xdd\xedg\xfb\xfe\
\xc3~\x9c$\xfevt\x9c\xa5\x18D6\x9dH~-\
\xf7\xf6G\xddR\xad\xb3\xb3\xecd\xa9IcqR\xf8\
\x8f\xdf\xff\xf4pg\xe7\xd9#\xafn6\xc7\xda\xd0\x14\
\xa6\xa9\x93\x5c$\x921\xf5g)\xa5\x09\xa6\xb5R\x05\
\xbfM.\xe5p\xaab\xba4[\xb3.\x8bF\x22u\
\xab\xc3\xa1\x92\xac\xab4\xebj\xd1\xa6\x07\xa0\xcf\xd5\xd4\
x%\x89q\x18\xf8!?\xe7\xadL\x14\xd5`\x11]\
\xcb\xaf:\xee\x80\x82\xad\x91h`F\x0d:I\xac\x0c\
\xedd#\xd5\xf1\x9e\xc7\x8e\xd3\x94o\xfef\xa8\xd3\xc3\
\xbb\xfd \xcd\xfa\xd1\x04\xc5\x89}|\xcb\xf8\x13\x82U\
\xf8}%\x8c\xda\xbd\xac\x13\xc8\xcd\xa3 m\xf5|\xfa\
\x97\xa6\x17>\xdfr\x0c\x9e\x84G\x9d/\xf9\xa7\xfab\
\xcbq\xb2\x84?@Q\xeb(H\x8e[\xed@\xaa(\
s\xea\x98\xc6c\xea\x19\x87\xad\xd9\xcc\x92\xa0\xef\xf6\xe2\
v\xe4U\x0e\x0eN=p\xcd\xe61\x1e\xd9I\x9d7\
5i\x93\x5c?\xb9\xb3^\x88fS;\xfba\xe7\xe0\
 W\xd3\x04\xf2\xd5\x90/O4yl\x868~h\
Y\xed\x09\x85g\x9c\xdcY^\xd6\xf1\x8d\xa7\xe4\xcc\x8f\
\xf0\x88w\xf4\xe9\x01\xfd\xd4\xc7\xc7R5\xd6b\xb2\x9a\
s\xaa!|\x8d&\xbeHjM\xc8l\xf4>m\xef\
\x7f\xf6^=\xc8\
\x00\x00\x04\x0b\
\x00\
\x00\x0b\x95x\x9c\xd5Vmo\x1bE\x10\x1e\xbf$n\
.%I\x9bR\x0a\x05\xb44\x12\xb2Ke.\xc5\xa5\
\x88\x90((\x01Z\xa9RJS*$\xcb:]\xce\
gg\xc1wg\xdd\xed\x99FU\xfe\x02\x1f\xf9)H\
|\xe6\xaf\xc0\x7f\xe0\x0b\x82\x99\xdd9\xdf\xc6qi%\
\xfa\xa5+\xado\xe7\xd9yyffo\xcf\x00\xb0\x04\
\x00u\x9c\x15\x9c+0\x01\x0f\x02H`\x843E\xa4\
\x86\x13\xfe\xe1\x01s\x06\x99]\x86\x01*\xfb0\x84=\
\xcb\xb4\xfe2\xa6\x8bp\x049\x9a\xd3\xca\x83M\xb8\x8b\
\xab}K\xa7\xaaw\x22\xe45\xc6\xd52\xa3\xbb\x96\xc6\
Z\xb1X\xc1\xb8cd\x11\x80\x04\x05'\x1c`\xb7`\
\xf2*\xc7\x22?\xfb\x16\xd1?j\xd5\x8d\x06\xc5t\xe0\
\x02\xbc\xcf\xe8%L\xa0\xa2\x89/\xea\xe77\x0f\x0e\x1f\
\xb43\xd5ow\xee\xb8\x86\xb3.\xb0\xde[\x85\x86f\
J3\xf2eL85\xe7\xa2N\xb2\xa6\xf1\x06\xf9\xc0\
\x15\xe1E\x0c\x92/Z\xf2=\x94Wy\xbd\xa0\xe5\x85\
\xa9\xbc1\xd5)\xb1\x06\x98\x22\x16X\x85\xf5v\xb5\xef\
\x9a\xc6\xaa\x1c\xe7\x12>oXqI\xfe\x80\xe5u\xd4\
\xaaj\xb9\xa6\xb3\xa2\xf5U\x5cQ\xb1\x04\xce\xb7P\xbf\
\xc1\xc5\xabk\xac\x8e\x952G\x8c\xf0-\x96\x97\x18#\
}\x87\xf5k\xac\xbf\xcc\xfc\x1c\xd6_\xe6\xfa\x10v\x0d\
\xe5\x15\xf6U\xd7\xf50uXa\x1fd\xbf\xc6\xbcV\
\xd9~M\xf7\xc8`o\xa2|\x99\xb9\x92\xbf\x8fX^\
g\x99\xec\xaf@q$\x01>\xc5zU\xa1<]\xc4\
\xf1/D\xa8\xe6\xdb\xcc\xfd\x0d\xe6\xf7%\xa2Wt\x0c\
\x13o\x9du\x16u\x8d\x0c\xfe3\xea8:ocw\
\xd5\xb2\xbb6\xc7\xeem\xc6\xbfE\x1d\x92\xdfa[\xb0\
\xb0\xeb\x8cU,\xec]\xc6\x88\xfbC\xac\x16\xd5\xeb=\
\xb6\xbf\xce\xfb\xe4{\x073Z\xe2\xbd\xbf\xd1\xc3g\xda\
\x0b\xd4\xf9l\x92\xd0\xe7'\x8d?7&a\x9a\xc9$\
\x16\x9b\xae\xeb\x8c\xd30\x90Z:\x96\xc3\xe3\xb1\x18\x8c\
\x12_m\x9d\x83e\x8c\xa0\x93\xa94\x0f\x948\xca\x07\
\xce3G\xe0\x88|\xd5\x11\xd1d\xbc\xa5%m+\x92\
\xb1\x1fHu\xb2\xe5\x9c\xa2E\x1e\xcbA\x92Fd\x22\
\xbc\xcd\xbb\x88L\xfc\xf4D\xc6C1\x09\x83O\xc4\xc4\
\x0b\x92Q\x92\x12\x9c\xc8\xbe \xc6\xcd\x16\xfb\x1e\x8e\xbc\
\xafS\x7f\xb8\xef+\xbf\xeb\xf6\xc46Yt\x9al!\
n\x92\xbb6\xc7\xbae\x0b-\x8c\xec83\x05x\x0a\
\xe5\xf8\xad,\xc0m\xf7uN\xea\x17+\xa9\xdf\xcb\xa4\
\xee\xbc\xa2\xa4\x92\x5cizb\x80\x8c\xf7LJ2~\
q\x8eS\xf5\xff\x91\x1e\x9d\xf9\xdbev\xd5\xef\x02\xe4\
5\x08SM\xefs\x91\x86C\x99\xa90m\x1e\xb9E\
\xd44\xf9\xc9\x8b\xfc\x1f0\x8cN\xae\xf3\xb4C\x01<\
L\x19\xf51\xca\x8f\xc9`\x90\x85\xaa\x19\xb8-\xbb\x06\
\xa4\xc3$f\xf4:-S\x18\xac\xa5\xafd\xc0n\xed\
b\xd8\x1bvI\xb8\xf8\x87\x0f\xef?z\xe2\xed\xa5I\
\x96y\xf7\xe3q\xae\x8a\xfa\x9c\xd1\xc7\xa8\x8f\xbf\xfa~\
\xef\xe0\xe0\xd1\xbek\x02\xce1?\xc8\xd5\x8c\xbdE\x04\
=\x1c>\xf1\x1e\xfb\xe90T\xecB7\x85\x14\xbc\xe7\
v\xc6x9\xdb\x1b\xcf\xee\x8dw\xa67\xe7\xd9\x98\xa6\
\x9fKR`U\x86\xa1'i]\xc4-\x82l\xdb\x9b\
\xedi\xc1\x0ajL\xd6\x00s\x02\x1a\xe3D\x0bF\xc9\
F\xdag\xb2+\xbb\xa4\x0fG\xa8\xf24\x9eqpj\
\x1f\xb8:\x7f\x00xT~\xdd\x90q0\xca\xfb\xa1\xf8\
\x22\x0a\x95?\xf2\xf0\xcf\xc0H\x1e\xed8%\x9e\xc9\xa8\
\xff1\xfd\xb4\x8fw\xf0\xfd\xc9\xe8\x02\x88\xfd(\xcc\xb0\
n\xa1\xd0V\xf3\xee\xcc\xe9\xf1\xfc\xaf\xb7\x91\xad\x88\x9c\
K\x84\x9f\xd7\xfanW\xd7\xb0\xe9\xb6z\xbd9\x962\
\x9e\x7f\xe6\xba\xdd<\xc3\xb7g\x94\x04qiI^\xa3\
0\xb6\xa2\x9aU\xb3\xf0\x85_\x01\xb4,Z\xd8\xeb\xdd\
\x12A\x12\xa3\x18\xeb\xf4>\xa4C\x83\xfb\xe6E\xd5\x8c\
Z\xd3\xcb\xa7pHs[<;5\x89'\xb3]3\
gR\xc6\xed\x97\xba2\xac\xce&\xa6\xa1\xd4\xd1\x05\x1d\
\x8d\xef\x90\x99{d\xfa\x11\x04\xfe\xaf;\xd3\xf6s\xfb\
\x056\xab\xa7\xc7\xbf\xc3\x1a`\x81\
\x00\x00\x03\xff\
\x00\
\x00\x0b\x1ax\x9c\xcdV[o\xdcD\x14\x9e\xbd\xa4K\
\x9c\x96\x86\x96$\xa5\xe52M@r\xca\xca\x98U\x8a\
\x10\x9b\x8d\x84\x12@\x95*\xa5\xedF-\xd2je\x19\
\xefdc\xc9k\xaf\xec\xf1\x92R\xe5/\xf0\xc8\x0b\x8f\
\xfc\x0c~\x03\xff\x87\x17\x04\xe7\xcc\x1c\xaf'\xdeM\x95\
GF\x9a\xf5\x9co\xce\xe5;\x17;a\x8c\xad2\xc6\
\x9a\xb0k\xb0\xdfa3\xe6\xb1\x1c~\x19\xab\xc3f\xff\
\xd2bK\x16\x1a\xdca\xa7,e>\x1b\xb3C\x96\xb0\
\x08vJ\xee\xdejZus\x83I&\xd89\x9c\xec\
\xeb\x98\x5cw\xdd\xa0\xe7\xa8\x84\x1a\xedF}\xa7\x85Q\
-\xc8\xf63B\xdf\x83l\x91\xc7\x1a\x98\xe0\xf3\x87\xa7\
\xfd\xa7N&G\xce\xdec\x17\xef\xdfe\x8d9\xd7\xdb\
\xac\xa5\x12\xc4=\xf1\xc3X\xdb3\xb6\x01{\x1d\xf4\x10\
o\xa1\x0f8\xdd\x82\xe76\xc5(\xe4\x87\x86\x8cv\x1f\
\x1b\xf2\x86!\xdf\x05Nu\xa5\xdfP\xd1\xf1\xbc\x09'\
L\x8a\xc3\xde\x02\xfd\x16%\xd9TX\x132\xd2\xb1q\
\x7f\x00\x9d\xb5\xe8\xbeV)L!?\x00\x7fk\xf0\xb4\
\xc8\xfe&\xe1\x88uI\xbeE\x18\xc6\xbbM\xfe\xea\xa4\
\xbf>\xaf\x89\xd6_\xa7Z\xd4\xe8\xfe}\xac8\xf1\xe9\
\x92\xbcA\x18\xfa\xdb$\x7f\x0d\x857\x81\xb3\xce\x0d\xd7\
\xe7$\xdf\xa7{\xf4\xf7\x80|\xa1\xcdWlE\xf1h\
\x16\x9d\x85\xfd7 +\x14\x0bk\xb1J\xfa=\x90\xd7\
T\x1fu>=\xca\xe5\x0e\xf1}\x05V-Us\xad\
\x83\xf8\x01x\x5c%\xacG\xb5\xde\x22\x9f\xc7\xe0\x1d\xb9\
\xdf#l\xcb\xa8+r\xfa\x16\xfc!\xd7\x0fI\xff>\
\xf9@\xde\x1f\x11\xfe+\xe8lR\xbf\xef\x11^\xd8}\
\xb2\xc4\x8e\x13\xfe\x1ctP~\xc8\xcaY)\xb0m\xc2\
j\x06\xb6C\x18\xf2z\x06Y`\x1e\x9f\x92\xfd6\xdd\
s\xcaw\x83\xee\xfe\x01\x0f_\xeb)i\xd2\x8c\xa30\
b\xe5\xe8\xfc\xb53\x13i\x16&1\xff\xd2u\xadi\
*\x82PIg\xe1\xf8l\xcaO\xa3\xc4\x97\xdd\x058\
\x8c\x01\xb4\xf28<M\xd2\x09a\x99?\x99F\x22\xed\
\x1cq)\xce\xe1v\xe6\xa7\xaf\xc3x\xccg\x22\xe8\xf0\
\x99\x97\xcf\x10K\xc2\x11G\x22\xf6\xae\xf5\xc6\xe2\xb0\xe0\
v\x8f\x07\xbc\x87F2OE\xe7\xc8\x86S[\x19\xec\
v\x95\xca8\xf2\xbeO\xfd\xf1\x91/\xfd\x81;\x04U\
\xb4\xb1\x03\xe7\xfc\xf5/\xfc\x11\x0f\x9c\x9f\xdb\xf8\x03\xca\
\x17\x96UI\xf4\x9c\x95\xeb\xf72\xd1\x8e[\x92\xff?\
\xd2\xfe\xcd\xa0\xfdGI\xfb\xf1[h\x87\xb1\xc98\xc9\
\xa5\xa6x\x0a\x0c\x0e\x93(I\xaf\x95\xc5B\x0es\xf3\
k\xd3\xc7\xe9\xec\x94\xec\xeb\xfb'E\x81\xf6\xd5,\xed\
\x1d`0\xfe\x0dO\xc58\xcc\xa4Hm\xe9\x82\x93\xbe\
N\xa7/})\xb8\x07\x1a\x1e%hjf\xa8ie\
\xa0\x13\x06z2\x8b|M\xecr\xd2\x99L\xf3@\xf2\
\xfe\xb3'/^z\x87i\x92e\xde\x93x\x9aK\xaa\
\x80\xe1\x05\x22\x9d|\xf7\xe3\xe1\xf1\xf1\x8b#\x17\xb2Z\
n{\x9c\xcb\x8a\xb1\x11\x0e<\xf4_z'~:\x16\
\x92\x5c\xa8\x9a\xa3\x82w\xa9\xf0dI\xa5wt\xf6\xb6\
\x99\xf7\x95m\xd0\xa6W4b\x91\xaan\xf8B\xfa\x1c\
*6\x16^\x88\xe7\xf94`\x11z\xe6\x8d\xa3\xab[\
P\xa0\x1c4\xb0$\x94\xb6L\x94\xa0\x95L\xc4\xb9\x94\
E\xd9\x22\xd4K\x05\xccH\x5cqpa\xce\x15\xfeu\
\xb8Y\xceU\xed\xcf\x9d0\x0e\xa2|$\xf8\xfeDH\
?\xf2\xe0\xaf|\x14\xfet`\x95x\x16NF_\xe0\
\x8fsv\x00/N\x86ou\xecOD6\xf5\x03\xc1\
\x95U\xd9d\x0c\xe3b\xe8\xabz;\x18\x04\xf8\xb4\xdd\
\xdd\xe1\xf0\xd2xh\xcb0^2Q\x83A\x9e\xc1\xdc\
FI\x10\x97f\xe8r\x22b#\xa4>\xd9\x85#\xf8\
\xba\x82e\xd1\x86\xe1\xb0=\xff\xc8\x8c\xf4;\xa4_\xa1\
\xc1\xa0xk\xd1w\xbb\xf8\x22\xe0]\x1fN\xc8\x98 \
\xa5Pt\xb9\x8c\x8a\xbb\xc7\xdf\x5ct\x97\x8d\xa46\xb5\
\x0bgm \xe5\x18\x13\x99T\xfby\xe5T\x1a\xedM\
tW\xb1\xad+\x8a\x06}/*\xdf\x8c\x85\xffr*\
\xbd_\xb8/\xb0\xaa\x9eZ\xff\x018\x196d\
\x00\x00\x04k\
\x00\
\x00\x0d\xafx\x9c\xe5Wmo\x1bE\x10^\xfb\x9c\x94\
\x5c\xc9K\x936i\xa1-\x9bZB6T\xc6\x09\x0e\
B5\x89\x82ZD+UJ\xd5\xa0\x0a\xc9\x8aN\xe7\
\xf39\x1c\xf8\xee\xac\xbb=\x93\xa8\xb2\xc4_\xe0\x0b?\
\x8c\xdf\xc0\x8f\xe0\x0b\x82\x99\xddY\xdf\xde\xd9)\xaaT\
\xf1\x85\x95\xd6\xdey\xd9y\x9e\x99\x9d\xdd8\x8c\xb1\x15\
\xa6F\x15\xa6\xcd<\x16\xb3\x11\xcc\x04$\x0bf\xe5o\
\x1al\xc1\xd8`cpMY\xc0\x04\xcc\x98E\xa0\xab\
\xa1\xe1M\x9b*0\xd7\xd8\x849sX\xff\xbam\x99\
\xf5Y\xc6\x86r\xe5\xb0}v\x00\xab'\x86OUZ\
B\x08>\x86\xd5u\xd2\x1e\x17\x18\xd3X\x03\xdc1s\
\x81\x03\x92\xbf$\x80c\x9d\xc0\xbb\x1a\x15\xfa\xd6A\x97\
\xe9{\x90\xbbX\xbfX\xd5\xfa5t\xb5\xd9{\xacN\
\xda\x1b\x90LE&\xb1,\xbf\xbf}~\xfa\xbc\x95\x8a\
A\xabs\xd0V\xfc\xad\x19\xc0:\x9d!\x82\x84n\x80\
\xa7 5X\x80-\x98\xf7q?XWh\xcdH\xbe\
N2\xc6x\xca\x96\x00S\x0d]8\xad\xab\x90\xce\xf4\
\xab\x92\xce\x92\xb1\xac\x99\xee)\xc4\xdd\xa1\xfdK\xe4\xaf\
\xe5\xba\x11W\xeb qy(ZW!\xbfc\x8a\xbb\
Cq\x91\xef\x1d\xf8~`\xf0Gy\xd7\x90\xef\x1b\xf9\
m\xc1\xae\xaa\xb4[\x92#\xae\xb7a\x85\x07\xc0a\xee\
\x80\xff5:\x10K\xeajP}\xb5F}\x97\xe4\x15\
\xc3n\x13?m\xb7\x8d\xba`\xbcU\x8a\x87\xe7p\x0b\
>\xd7\x08\x0b\xc7\xa7$\xaf\x93\xff]\x907\xc8\x7f]\
\xf2Vu\xd516\x08s\x93\xf0o\x10\xe6&\x9d\xa9\
E\x187\x09\xa3B\x187\xa5^\x8d\xdb oS\xcc\
\x9a\xc4Pg\xb3M\x18\x18\xff6\xd5f\x87\xe2\xa3|\
\x87th\xff`V;%\xdf#\xacU\xf2\xbf\xc7\xf2\
\x1eB\xfb.qC\xfb\x17\x90S\x95\xe5\xcd\x8f\xfa?\
A\x83}qH\xf5\x7f\x9fjx\x04\xd6\x15\x92\xbf\x06\
\x0f\xc4\xfd\x90\xb8\xdc\x22\xffmY7\xa5?\xa4z\x7f\
D\xf8\xbf\xc2\x9eU\xaa\xc5]\xd2c\x9c]\xea\x99-\
\x8a\x838\x0f\xc8\xef/`\xfd\xa5\xba\xa25\xba7(\
\x0cX~o\xff\xa8O\xfc$\x0d\xe2\x88\xef\xb5\xdb\xb6\
\x9d\x8a$\xf3\x04\xefgC\xfb\xb5\xcda\x84\xae\xe8\xf0\
p2\xeeJi8\x8a]\xc1\xe3\xb1\xeb\x05\xe2\xb2k\
O\xbb\xb6\x9dE\xc10NB\xdc\xc2\x9d\xfd\x03\xd0L\
\xdc\xe42\x88\xce\xf9\xc4\xf7>\xe7\x13\xc7\x8bGq\xd2\
\xb5]!\x92\xa0\x9f\x09_\xe9\x17i;|\x1c\xa7\x81\
\x002\x18%\x0e\x06\x1cI7\x9aD\x85B\xf1C\xbd\
\x19\x95\xe7#\xe7\x05m\x02\x03\x10h\x01Y\xfe\x89\x11\
ij\xdb\xa5\xfc/X>\x8c\xfc\xf7\xff/\xf9\xfff\
\xe4\xff{\x9e\xff\xc1;\xca?\xceD)\xf7 *$\
\xad\xc4\xff\x22[\xbc\x9c\xfby\xb2\xd6\xa6\x074\x87~\
\x22\xd9>\xe2\x89\x7f\x1e\xa4\xc2O\x1a\xfd\xb6\x86M\xe2\
\x9f\x9d\xd0\xfd\x11\x80e\xae\x9d\x8b\x0e\xc28\x08\xf3\x88\
C\xde?\xc5\xc3a\xea\x8b\x86\xd7n\x9a%A\x1f*\
K\xc9\xaf\xd3Tu\x82\xd2\xba\x22\xf0(\xac\x99I\xb7\
`2\xcaVT/P\x16JHGw\xfa\xe2\xd9\xcb\
W\xce\xe3$NS\xe7Y4\xce\x04%V\xda\x00,\
\xbf\xfb\xe6\xfb\xc7''/\x9f\xb4\x8dD\x08\xc6\xb0\xee\
)\xfa\x0b\xa2\x9fd\xa2\x14~\xc6\xfd\x8a\xe8\x85\xb4\xc1\
\xe7\xf4\x95Q\x84\xa9n\x03hH\xe1\xbcm/\x84\xd9\
\xa8\xa1s{\xa8\x8f\xac)\xfba\x9e\xb3\xea\xb4\xb9J\
q\xa8\xed\xb9\xef\x04\xb8\xd6\xd0\x1a\xd80\xb5\x0c\x12\xe3\
\x9c\x81\xe9\x91\x1f\x8b\xa4\x9f'\xa4\x14\x0b\x18\xa9\xdd\xb1\
\x14\x94\x93\xa9i\x15\x93-4\xcf\x9co^\xadY'\
\xc9\xce\xf6E\x96D%\xa0\xa9y[j\xf4\xe7\x89F\
u\xa9\x1eD\xde(\x1b\xf8\xfc\xab\xd0\x17\xee\xc8\x81\x9f\
g\xa3\xa0\x7fd\xe7\xfa4\x08\x07\x9f\xe1G\xeb\x87#\
x\x0bR|\xf7\x227\xf4S\xb8\x02>\x97\xbb\xba\x0b\
^\x95\xd9\xddz\xd3\xcbB\xbb\x90\x5c\x1b\x09/\xee\xb4\
^/K\xe1\x06\x8fb/j7\xcf\xce\xae\xec\xb6^\
O\x1f\x0a:\xcd\xc5\x0f\xa2+\xeeI\xaf7{\xa3\x1b\
%\x00}YL\x97\xbd\xa6\x0e\x8f\xa7\xee_\xe4\xf4\xd5\
\xaa\xa1\xe1x\x80\xb1u\xd3\x9c\x9d=\x84`\x11\x88\x91\
\xac\xd3\xc7\xd8\xc2`W\xcf\x95\x04n\xce^d\x1d\x10\
\xe7!\x7f=U\x94\xe2\xc2\xd1\x07\x91\xd9\xa6\xf1\x5c\x0b\
\xe5o'x\x16\xdb\x95\x1a%V\xfd\x81\x0d\xb2$1\
\xe9=-\xbd\xa9\xb3_\x12\x8c\xfe\xb1)u\xd1\x9c]\
\xeb\xca~r\xfc\x03\x0d.\xdb\xc1\
"

qt_resource_name = b"\
\x00\x0d\
\x0d\xf9\xb2\xf2\
\x00q\
\x00u\x00a\x00d\x00.\x00v\x00e\x00r\x00t\x00.\x00q\x00s\x00b\
\x00\x0e\
\x04\x16\xeb\xb2\
\x00c\
\x00o\x00l\x00o\x00r\x00.\x00f\x00r\x00a\x00g\x00.\x00q\x00s\x00b\
\x00\x0d\
\x09\x18\xb0\xd2\
\x00q\
\x00u\x00a\x00d\x00.\x00f\x00r\x00a\x00g\x00.\x00q\x00s\x00b\
\x00\x0e\
\x00\xfb\xe9\x92\
\x00c\
\x00o\x00l\x00o\x00r\x00.\x00v\x00e\x00r\x00t\x00.\x00q\x00s\x00b\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x04\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00b\x00\x00\x00\x00\x00\x01\x00\x00\x0b\xec\
\x00\x00\x01\x8b8N2\x22\
\x00\x00\x00 \x00\x00\x00\x00\x00\x01\x00\x00\x03\xda\
\x00\x00\x01\x8b8N2\x22\
\x00\x00\x00B\x00\x00\x00\x00\x00\x01\x00\x00\x07\xe9\
\x00\x00\x01\x8b8N2\x22\
\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x8b8N2\x22\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Property, QObject
from PySide6.QtQml import QQmlComponent


class WithComponent(QObject):
    def get_component(self):
        return None

    component = Property(QQmlComponent, fget=get_component)


class TestQmlSupport(unittest.TestCase):

    def testMetatypeValid(self):
        m = WithComponent.staticMetaObject
        c = m.property(m.indexOfProperty("component"))

        self.assertTrue(c.typeId() > 0)
        self.assertTrue(c.typeName() == "QQmlComponent*")
        self.assertTrue(c.metaType().isValid())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/layouts/dynamiclayouts example from Qt v5.x"""

from PySide6.QtCore import Qt, QSize
from PySide6.QtWidgets import (QApplication, QDialog, QLayout, QGridLayout,
                               QMessageBox, QGroupBox, QSpinBox, QSlider,
                               QProgressBar, QDial, QDialogButtonBox,
                               QComboBox, QLabel)


class Dialog(QDialog):
    def __init__(self):
        super().__init__()

        self._rotable_widgets = []

        self.create_rotable_group_box()
        self.create_options_group_box()
        self.create_button_box()

        main_layout = QGridLayout()
        main_layout.addWidget(self._rotable_group_box, 0, 0)
        main_layout.addWidget(self._options_group_box, 1, 0)
        main_layout.addWidget(self._button_box, 2, 0)
        main_layout.setSizeConstraint(QLayout.SizeConstraint.SetMinimumSize)

        self._main_layout = main_layout
        self.setLayout(self._main_layout)

        self.setWindowTitle("Dynamic Layouts")

    def rotate_widgets(self):
        count = len(self._rotable_widgets)
        if count % 2 == 1:
            raise AssertionError("Number of widgets must be even")

        for widget in self._rotable_widgets:
            self._rotable_layout.removeWidget(widget)

        self._rotable_widgets.append(self._rotable_widgets.pop(0))

        for i in range(count // 2):
            self._rotable_layout.addWidget(self._rotable_widgets[count - i - 1], 0, i)
            self._rotable_layout.addWidget(self._rotable_widgets[i], 1, i)

    def buttons_orientation_changed(self, index):
        self._main_layout.setSizeConstraint(QLayout.SetNoConstraint)
        self.setMinimumSize(0, 0)

        orientation = Qt.Orientation(self._buttons_orientation_combo_box.itemData(index))

        if orientation == self._button_box.orientation():
            return

        self._main_layout.removeWidget(self._button_box)

        spacing = self._main_layout.spacing()

        old_size_hint = self._button_box.sizeHint() + QSize(spacing, spacing)
        self._button_box.setOrientation(orientation)
        new_size_hint = self._button_box.sizeHint() + QSize(spacing, spacing)

        if orientation == Qt.Orientation.Horizontal:
            self._main_layout.addWidget(self._button_box, 2, 0)
            self.resize(self.size() + QSize(-old_size_hint.width(), new_size_hint.height()))
        else:
            self._main_layout.addWidget(self._button_box, 0, 3, 2, 1)
            self.resize(self.size() + QSize(new_size_hint.width(), -old_size_hint.height()))

        self._main_layout.setSizeConstraint(QLayout.SetDefaultConstraint)

    def show_help(self):
        QMessageBox.information(self, "Dynamic Layouts Help",
                                "This example shows how to change layouts "
                                "dynamically.")

    def create_rotable_group_box(self):
        self._rotable_group_box = QGroupBox("Rotable Widgets")

        self._rotable_widgets.append(QSpinBox())
        self._rotable_widgets.append(QSlider())
        self._rotable_widgets.append(QDial())
        self._rotable_widgets.append(QProgressBar())
        count = len(self._rotable_widgets)
        for i in range(count):
            element = self._rotable_widgets[(i + 1) % count]
            self._rotable_widgets[i].valueChanged[int].connect(element.setValue)

        self._rotable_layout = QGridLayout()
        self._rotable_group_box.setLayout(self._rotable_layout)

        self.rotate_widgets()

    def create_options_group_box(self):
        self._options_group_box = QGroupBox("Options")

        buttons_orientation_label = QLabel("Orientation of buttons:")

        buttons_orientation_combo_box = QComboBox()
        buttons_orientation_combo_box.addItem("Horizontal", Qt.Orientation.Horizontal)
        buttons_orientation_combo_box.addItem("Vertical", Qt.Orientation.Vertical)
        buttons_orientation_combo_box.currentIndexChanged[int].connect(
            self.buttons_orientation_changed)

        self._buttons_orientation_combo_box = buttons_orientation_combo_box

        options_layout = QGridLayout()
        options_layout.addWidget(buttons_orientation_label, 0, 0)
        options_layout.addWidget(self._buttons_orientation_combo_box, 0, 1)
        options_layout.setColumnStretch(2, 1)
        self._options_group_box.setLayout(options_layout)

    def create_button_box(self):
        self._button_box = QDialogButtonBox()

        close_button = self._button_box.addButton(QDialogButtonBox.StandardButton.Close)
        help_button = self._button_box.addButton(QDialogButtonBox.StandardButton.Help)
        rotate_widgets_button = self._button_box.addButton(
            "Rotate &Widgets", QDialogButtonBox.ButtonRole.ActionRole)

        rotate_widgets_button.clicked.connect(self.rotate_widgets)
        close_button.clicked.connect(self.close)
        help_button.clicked.connect(self.show_help)


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)
    dialog = Dialog()
    dialog.exec()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QListWidget, QListWidgetItem
from helper.usesqapplication import UsesQApplication


class QListWidgetItemConstructor(UsesQApplication):

    def setUp(self):
        super(QListWidgetItemConstructor, self).setUp()
        self.widgetList = QListWidget()

    def tearDown(self):
        del self.widgetList
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(QListWidgetItemConstructor, self).tearDown()

    def testConstructorWithParent(self):
        # Bug 235 - QListWidgetItem constructor not saving ownership
        QListWidgetItem(self.widgetList)
        item = self.widgetList.item(0)
        self.assertEqual(item.listWidget(), self.widgetList)

    def testConstructorWithNone(self):
        # Bug 452 - QListWidgetItem() not casting NoneType to null correctly.
        item = QListWidgetItem(None, 123)  # noqa: F841


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""Tests covering signal emission and receiving to python slots"""

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QSpinBox, QPushButton

from helper.basicpyslotcase import BasicPySlotCase
from helper.usesqapplication import UsesQApplication


class ButtonPySlot(UsesQApplication, BasicPySlotCase):
    """Tests the connection of python slots to QPushButton signals"""

    def testButtonClicked(self):
        """Connection of a python slot to QPushButton.clicked()"""
        button = QPushButton('Mylabel')
        button.clicked.connect(self.cb)
        self.args = tuple()
        button.clicked.emit()
        self.assertTrue(self.called)

    def testButtonClick(self):
        """Indirect qt signal emission using the QPushButton.click() method """
        button = QPushButton('label')
        button.clicked.connect(self.cb)
        self.args = tuple()
        button.click()
        self.assertTrue(self.called)


class SpinBoxPySlot(UsesQApplication, BasicPySlotCase):
    """Tests the connection of python slots to QSpinBox signals"""

    def setUp(self):
        super(SpinBoxPySlot, self).setUp()
        self.spin = QSpinBox()

    def tearDown(self):
        del self.spin
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(SpinBoxPySlot, self).tearDown()

    def testSpinBoxValueChanged(self):
        """Connection of a python slot to QSpinBox.valueChanged(int)"""
        self.spin.valueChanged.connect(self.cb)
        self.args = [3]
        self.spin.valueChanged.emit(*self.args)
        self.assertTrue(self.called)

    def testSpinBoxValueChangedImplicit(self):
        """Indirect qt signal emission using QSpinBox.setValue(int)"""
        self.spin.valueChanged.connect(self.cb)
        self.args = [42]
        self.spin.setValue(self.args[0])
        self.assertTrue(self.called)

    def atestSpinBoxValueChangedFewArgs(self):
        """Emission of signals with fewer arguments than needed"""
        self.spin.valueChanged.connect(self.cb)
        self.args = (554,)
        self.assertRaises(TypeError, self.spin.valueChanged.emit)


class QSpinBoxQtSlots(UsesQApplication):
    """Tests the connection to QSpinBox qt slots"""

    qapplication = True

    def testSetValueIndirect(self):
        """Indirect signal emission: QSpinBox using valueChanged(int)/setValue(int)"""
        spinSend = QSpinBox()
        spinRec = QSpinBox()

        spinRec.setValue(5)

        spinSend.valueChanged.connect(spinRec.setValue)
        self.assertEqual(spinRec.value(), 5)
        spinSend.setValue(3)
        self.assertEqual(spinRec.value(), 3)
        self.assertEqual(spinSend.value(), 3)

    def testSetValue(self):
        """Direct signal emission: QSpinBox using valueChanged(int)/setValue(int)"""
        spinSend = QSpinBox()
        spinRec = QSpinBox()

        spinRec.setValue(5)
        spinSend.setValue(42)

        spinSend.valueChanged.connect(spinRec.setValue)
        self.assertEqual(spinRec.value(), 5)
        self.assertEqual(spinSend.value(), 42)
        spinSend.valueChanged.emit(3)

        self.assertEqual(spinRec.value(), 3)
        # Direct emission shouldn't change the value of the emitter
        self.assertEqual(spinSend.value(), 42)

        spinSend.valueChanged.emit(66)
        self.assertEqual(spinRec.value(), 66)
        self.assertEqual(spinSend.value(), 42)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

import sys

from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine

from autogen.settings import setup_qt_environment
from audio import *  # noqa: F401,F403


def main():
    app = QGuiApplication(sys.argv)
    engine = QQmlApplicationEngine()

    setup_qt_environment(engine)

    if not engine.rootObjects():
        sys.exit(-1)

    ex = app.exec()
    del engine
    return ex


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from minimal import Obj


class ExtObj(Obj):
    def __init__(self, objId):
        Obj.__init__(self, objId)
        self.virtual_method_called = False

    def virtualMethod(self, val):
        self.virtual_method_called = True
        return not Obj.virtualMethod(self, val)

    def passObjectType(self, obj):
        obj.setObjId(obj.objId() + 1)
        return obj

    def passObjectTypeReference(self, obj):
        obj.setObjId(obj.objId() + 1)
        return obj


class ObjTest(unittest.TestCase):

    def testNormalMethod(self):
        objId = 123
        obj = Obj(objId)
        self.assertEqual(obj.objId(), objId)

    def testNormalMethodFromExtendedClass(self):
        objId = 123
        obj = ExtObj(objId)
        self.assertEqual(obj.objId(), objId)

    def testVirtualMethod(self):
        obj = Obj(0)
        even_number = 8
        self.assertEqual(obj.virtualMethod(even_number), obj.callVirtualMethod(even_number))

    def testVirtualMethodFromExtendedClass(self):
        obj = ExtObj(0)
        even_number = 8
        self.assertEqual(obj.virtualMethod(even_number), obj.callVirtualMethod(even_number))
        self.assertTrue(obj.virtual_method_called)

    def testPassObjectType(self):
        obj = Obj(0)
        self.assertEqual(obj, obj.passObjectType(obj))
        self.assertEqual(obj, obj.callPassObjectType(obj))

    def testPassObjectTypeNone(self):
        obj = Obj(0)
        self.assertEqual(None, obj.passObjectType(None))
        self.assertEqual(None, obj.callPassObjectType(None))

    def testPassObjectTypeReference(self):
        obj = Obj(0)
        self.assertEqual(obj, obj.passObjectTypeReference(obj))
        self.assertEqual(obj, obj.callPassObjectTypeReference(obj))

    def testPassObjectTypeFromExtendedClass(self):
        obj = ExtObj(0)
        self.assertEqual(obj.objId(), 0)
        sameObj = obj.passObjectType(obj)
        self.assertEqual(obj, sameObj)
        self.assertEqual(sameObj.objId(), 1)
        sameObj = obj.callPassObjectType(obj)
        self.assertEqual(obj, sameObj)
        self.assertEqual(sameObj.objId(), 2)

    def testPassObjectTypeReferenceFromExtendedClass(self):
        obj = ExtObj(0)
        self.assertEqual(obj.objId(), 0)
        sameObj = obj.passObjectTypeReference(obj)
        self.assertEqual(obj, sameObj)
        self.assertEqual(sameObj.objId(), 1)
        sameObj = obj.callPassObjectTypeReference(obj)
        self.assertEqual(obj, sameObj)
        self.assertEqual(sameObj.objId(), 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""Tests covering signal emission and receiving to python slots"""

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal, Slot, SIGNAL
from helper.usesqapplication import UsesQApplication


class Sender(QObject):

    foo = Signal()
    foo2 = Signal()


class MyObject(QObject):

    foo2 = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._slotCalledCount = 0

    # this '@Slot()' is needed to get the right sort order in testSharedSignalEmission.
    # For some reason, it also makes the tests actually work!
    @Slot()
    def mySlot(self):
        self._slotCalledCount = self._slotCalledCount + 1


class StaticMetaObjectTest(UsesQApplication):

    def testSignalPropagation(self):
        """Old style, dynamic signal creation."""
        o = QObject()
        o2 = MyObject()

        # SIGNAL foo not created yet
        self.assertEqual(o.metaObject().indexOfSignal("foo()"), -1)

        o.connect(SIGNAL("foo()"), o2.mySlot)
        # SIGNAL foo create after connect
        self.assertTrue(o.metaObject().indexOfSignal("foo()") > 0)

        # SIGNAL does not propagate to others objects of the same type
        self.assertEqual(o2.metaObject().indexOfSignal("foo()"), -1)

        del o
        del o2
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        o = MyObject()
        # The SIGNAL was destroyed with old objects
        self.assertEqual(o.metaObject().indexOfSignal("foo()"), -1)

    def testSharedSignalEmission(self):
        o = Sender()
        m = MyObject()

        o.foo2.connect(m.mySlot)
        m.foo2.connect(m.mySlot)
        o.foo2.emit()
        self.assertEqual(m._slotCalledCount, 1)
        del o
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        m.foo2.emit()
        self.assertEqual(m._slotCalledCount, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for modified virtual methods.'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import VirtualMethods, Str


class ExtendedVirtualMethods(VirtualMethods):
    def __init__(self):
        VirtualMethods.__init__(self)
        self.name_called = False
        self.sum0_called = False
        self.sum1_called = False
        self.sum2_called = False
        self.sum3_called = False
        self.sum4_called = False
        self.sumThree_called = False
        self.callMe_called = 0
        self.multiplier = 12345

    def sum0(self, a0, a1, a2):
        self.sum0_called = True
        return VirtualMethods.sumThree(self, a0, a1, a2) * self.multiplier

    def sumThree(self, a0, a1, a2):
        self.sumThree_called = True
        return VirtualMethods.sumThree(self, a0, a1, a2) * self.multiplier

    def sum1(self, a0, a1, a2):
        self.sum1_called = True
        return VirtualMethods.sum1(self, a0, a1, a2) * self.multiplier

    def sum2(self, a0, a1):
        self.sum2_called = True
        return VirtualMethods.sum2(self, a0, a1) * self.multiplier

    def sum3(self, a0, a1):
        self.sum3_called = True
        return VirtualMethods.sum3(self, a0, a1) * self.multiplier

    def sum4(self, a0, a1):
        self.sum4_called = True
        return VirtualMethods.sum4(self, a0, a1) * self.multiplier

    def name(self):
        self.name_called = True
        return Str('ExtendedVirtualMethods')

    def callMe(self):
        self.callMe_called += 1

    def getMargins(self):
        return tuple([m * 2 for m in VirtualMethods.getMargins(self)])


class VirtualMethodsTest(unittest.TestCase):
    '''Test case for modified virtual methods.'''

    def setUp(self):
        self.vm = VirtualMethods()
        self.evm = ExtendedVirtualMethods()

    def tearDown(self):
        del self.vm
        del self.evm
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testModifiedVirtualMethod0(self):
        '''Renamed virtual method.'''
        a0, a1, a2 = 2, 3, 5
        result0 = self.vm.callSum0(a0, a1, a2)
        result1 = self.vm.sumThree(a0, a1, a2)
        self.assertEqual(result0, a0 + a1 + a2)
        self.assertEqual(result0, result1)
        self.assertRaises(AttributeError, getattr, self.vm, 'sum0')

    def testReimplementedModifiedVirtualMethod0(self):
        '''Override of a renamed virtual method.'''
        a0, a1, a2 = 2, 3, 5
        result0 = self.vm.callSum0(a0, a1, a2)
        result1 = self.vm.sumThree(a0, a1, a2)
        result2 = self.evm.callSum0(a0, a1, a2)
        self.assertEqual(result0, result1)
        self.assertEqual(result0 * self.evm.multiplier, result2)
        self.assertTrue(self.evm.sumThree_called)
        self.assertFalse(self.evm.sum0_called)

    def testModifiedVirtualMethod1(self):
        '''Virtual method with three arguments and the last one
        changed to have the default value set to 1000.'''
        a0, a1, a2 = 2, 3, 5
        result0 = self.vm.sum1(a0, a1)
        self.assertEqual(result0, a0 + a1 + 1000)
        result1 = self.vm.sum1(a0, a1, a2)
        result2 = self.vm.callSum1(a0, a1, a2)
        self.assertEqual(result1, result2)

    def testReimplementedModifiedVirtualMethod1(self):
        '''Override of the virtual method with three arguments and
        the last one changed to have the default value set to 1000.'''
        a0, a1 = 2, 3
        result0 = self.vm.sum1(a0, a1)
        result1 = self.evm.callSum1(a0, a1, 1000)
        self.assertEqual(result0 * self.evm.multiplier, result1)
        self.assertTrue(self.evm.sum1_called)

    def testModifiedVirtualMethod2(self):
        '''Virtual method originally with three arguments, the last
        one was removed and the default value set to 2000.'''
        a0, a1 = 1, 2
        result0 = self.vm.sum2(a0, a1)
        self.assertEqual(result0, a0 + a1 + 2000)
        result1 = self.vm.sum2(a0, a1)
        result2 = self.vm.callSum2(a0, a1, 2000)
        self.assertEqual(result1, result2)
        self.assertRaises(TypeError, self.vm.sum2, 1, 2, 3)

    def testReimplementedModifiedVirtualMethod2(self):
        '''Override of the virtual method originally with three arguments,
        the last one was removed and the default value set to 2000.'''
        a0, a1 = 1, 2
        ignored = 54321
        result0 = self.vm.sum2(a0, a1)
        result1 = self.evm.callSum2(a0, a1, ignored)
        self.assertEqual(result0 * self.evm.multiplier, result1)
        self.assertTrue(self.evm.sum2_called)

    def testModifiedVirtualMethod3(self):
        '''Virtual method originally with three arguments have the second
        one removed and replaced by custom code that replaces it by the sum
        of the first and the last arguments.'''
        a0, a1 = 1, 2
        result0 = self.vm.sum3(a0, a1)
        self.assertEqual(result0, a0 + (a0 + a1) + a1)
        result1 = self.vm.callSum3(a0, 10, a1)
        self.assertNotEqual(result0, result1)
        result2 = self.vm.callSum3(a0, a0 + a1, a1)
        self.assertEqual(result0, result2)
        self.assertRaises(TypeError, self.vm.sum3, 1, 2, 3)

    def testReimplementedModifiedVirtualMethod3(self):
        '''Override of the virtual method originally with three arguments
        have the second one removed and replaced by custom code that
        replaces it by the sum of the first and the last arguments.'''
        a0, a1 = 1, 2
        ignored = 54321
        result0 = self.vm.sum3(a0, a1)
        result1 = self.evm.callSum3(a0, ignored, a1)
        self.assertEqual(result0 * self.evm.multiplier, result1)
        self.assertTrue(self.evm.sum3_called)

    def testModifiedVirtualMethod4(self):
        '''Virtual method originally with three arguments, the
        last one was removed and the default value set to 3000.'''
        a0, a1 = 1, 2
        default_value = 3000
        result0 = self.vm.sum4(a0, a1)
        self.assertEqual(result0, a0 + default_value + a1)
        removed_arg_value = 100
        result1 = self.vm.callSum4(a0, removed_arg_value, a1)
        self.assertEqual(result1, a0 + removed_arg_value + a1)
        self.assertRaises(TypeError, self.vm.sum4, 1, 2, 3)

    def testReimplementedModifiedVirtualMethod4(self):
        '''Override of the virtual method originally with three arguments,
        the last one was removed and the default value set to 3000.
        The method was modified with code injection on the binding override
        (the one that receives calls from C++ with the original signature
        and forwards it to Python overrides) that subtracts the value of the
        second argument (removed in Python) from the value of the first
        before sending them to Python.'''
        a0, a1 = 1, 2
        removed_arg_value = 2011
        default_value = 3000
        result = self.evm.callSum4(a0, removed_arg_value, a1)
        self.assertEqual(result,
                         (a0 - removed_arg_value + a1 + default_value) * self.evm.multiplier)
        self.assertTrue(self.evm.sum4_called)

    def testOverridenMethodResultModification(self):
        '''Injected code modifies the result of a call to a virtual
        method overridden in Python.'''
        orig_name = self.vm.callName()
        self.assertEqual(orig_name, 'VirtualMethods')
        name = self.evm.callName()
        self.assertEqual(name, 'PimpedExtendedVirtualMethods')
        self.assertEqual(name, Str('PimpedExtendedVirtualMethods'))
        self.assertTrue(self.evm.name_called)

    def testInjectCodeCallsPythonVirtualMethodOverride(self):
        '''When injected code calls the Python override by itself
        no code for the method call should be generated.'''
        self.evm.callCallMe()
        self.assertEqual(self.evm.callMe_called, 1)

    def testAllArgumentsRemoved(self):
        values = (10, 20, 30, 40)
        self.vm.setMargins(*values)
        self.assertEqual(self.vm.getMargins(), values)

    def testAllArgumentsRemovedCallVirtual(self):
        values = (10, 20, 30, 40)
        self.vm.setMargins(*values)
        self.assertEqual(self.vm.callGetMargins(), values)

    def testExtendedAllArgumentsRemoved(self):
        values = (10, 20, 30, 40)
        self.evm.setMargins(*values)
        double = tuple([m * 2 for m in values])
        self.assertEqual(self.evm.getMargins(), double)

    def testExtendedAllArgumentsRemovedCallVirtual(self):
        values = (10, 20, 30, 40)
        self.evm.setMargins(*values)
        double = tuple([m * 2 for m in values])
        self.assertEqual(self.evm.callGetMargins(), double)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2010 velociraptor Genjix <aphidia@hotmail.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import (QEvent, QPoint, QRandomGenerator, QSize, Qt,
                            Property)
from PySide6.QtGui import QFont, QFontMetrics, QFontDatabase, QPainter
from PySide6.QtWidgets import QApplication, QMainWindow
from PySide6.QtStateMachine import (QEventTransition, QFinalState,
                                    QKeyEventTransition, QState, QStateMachine)


class MovementTransition(QEventTransition):
    def __init__(self, window):
        super().__init__(window, QEvent.Type.KeyPress)
        self.window = window

    def eventTest(self, event):
        if (event.type() == QEvent.Type.StateMachineWrapped
                and event.event().type() == QEvent.Type.KeyPress):
            key = event.event().key()
            return (key == Qt.Key.Key_2 or key == Qt.Key.Key_8
                    or key == Qt.Key.Key_6 or key == Qt.Key.Key_4)
        return False

    def onTransition(self, event):
        key = event.event().key()
        if key == Qt.Key.Key_4:
            self.window.move_player(self.window.left)
        if key == Qt.Key.Key_8:
            self.window.move_player(self.window.Up)
        if key == Qt.Key.Key_6:
            self.window.move_player(self.window.right)
        if key == Qt.Key.Key_2:
            self.window.move_player(self.window.down)


class Custom(QState):
    def __init__(self, parent, mw):
        super().__init__(parent)
        self.mw = mw

    def onEntry(self, e):
        print(self.mw.status)


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.pX = 5
        self.pY = 5
        self.width = 35
        self.height = 20
        self._status_str = ''

        font = QFont()
        if 'Monospace' in QFontDatabase.families():
            font = QFont('Monospace', 12)
        else:
            for family in QFontDatabase.families():
                if QFontDatabase.isFixedPitch(family):
                    font = QFont(family, 12)
        self.setFont(font)

        self.setup_map()
        self.build_machine()
        self.show()

    def setup_map(self):
        self.map = []
        generator = QRandomGenerator().global_()
        for x in range(self.width):
            column = []
            for y in range(self.height):
                if (x == 0 or x == self.width - 1 or y == 0
                        or y == self.height - 1 or generator.bounded(0, 40) == 0):
                    column.append('#')
                else:
                    column.append('.')
            self.map.append(column)

    def build_machine(self):
        machine = QStateMachine(self)

        input_state = Custom(machine, self)
        # this line sets the status
        self.status = 'hello!'
        # however this line does not
        input_state.assignProperty(self, 'status', 'Move the rogue with 2, 4, 6, and 8')

        machine.setInitialState(input_state)
        machine.start()

        transition = MovementTransition(self)
        input_state.addTransition(transition)

        quit_state = QState(machine)
        quit_state.assignProperty(self, 'status', 'Really quit(y/n)?')

        yes_transition = QKeyEventTransition(self, QEvent.Type.KeyPress, Qt.Key.Key_Y)
        self._final_state = QFinalState(machine)
        yes_transition.setTargetState(self._final_state)
        quit_state.addTransition(yes_transition)

        no_transition = QKeyEventTransition(self, QEvent.Type.KeyPress, Qt.Key.Key_N)
        no_transition.setTargetState(input_state)
        quit_state.addTransition(no_transition)

        quit_transition = QKeyEventTransition(self, QEvent.Type.KeyPress, Qt.Key.Key_Q)
        quit_transition.setTargetState(quit_state)
        input_state.addTransition(quit_transition)

        machine.setInitialState(input_state)
        machine.finished.connect(qApp.quit)  # noqa: F821
        machine.start()

    def sizeHint(self):
        metrics = QFontMetrics(self.font())
        return QSize(metrics.horizontalAdvance('X') * self.width,
                     metrics.height() * (self.height + 1))

    def paintEvent(self, event):
        metrics = QFontMetrics(self.font())
        with QPainter(self) as painter:
            font_height = metrics.height()
            font_width = metrics.horizontalAdvance('X')

            painter.fillRect(self.rect(), Qt.GlobalColor.black)
            painter.setPen(Qt.GlobalColor.white)

            y_pos = font_height
            painter.drawText(QPoint(0, y_pos), self.status)
            for y in range(self.height):
                y_pos += font_height
                x_pos = 0
                for x in range(self.width):
                    if y == self.pY and x == self.pX:
                        x_pos += font_width
                        continue
                    painter.drawText(QPoint(x_pos, y_pos), self.map[x][y])
                    x_pos += font_width
            painter.drawText(QPoint(self.pX * font_width, (self.pY + 2) * font_height), '@')

    def move_player(self, direction):
        if direction == self.left:
            if self.map[self.pX - 1][self.pY] != '#':
                self.pX -= 1
        elif direction == self.right:
            if self.map[self.pX + 1][self.pY] != '#':
                self.pX += 1
        elif direction == self.Up:
            if self.map[self.pX][self.pY - 1] != '#':
                self.pY -= 1
        elif direction == self.down:
            if self.map[self.pX][self.pY + 1] != '#':
                self.pY += 1
        self.repaint()

    def get_status(self):
        return self._status_str

    def set_status(self, status):
        self._status_str = status
        self.repaint()
    status = Property(str, get_status, set_status)
    Up = 0
    down = 1
    left = 2
    right = 3
    width = 35
    height = 20


if __name__ == '__main__':
    app = QApplication(sys.argv)
    main_win = MainWindow()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QVariant::Type converter'''
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QMetaType
from PySide6.QtSql import QSqlField


class QVariantTypeTest(unittest.TestCase):
    def testQVariantType(self):
        cmp_id = QMetaType.QString.value

        f = QSqlField("name", QMetaType(QMetaType.QString))
        self.assertEqual(f.metaType().id(), cmp_id)

        f = QSqlField("name", QMetaType.fromName(b"QString"))
        self.assertEqual(f.metaType().id(), cmp_id)

        f = QSqlField("name", QMetaType.fromName(b"double"))
        self.assertEqual(f.metaType(), float)

        f = QSqlField("name", float)
        self.assertEqual(f.metaType(), float)

        f = QSqlField("name", int)
        self.assertEqual(f.metaType(), int)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for Array types (PySequence).'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
import sample


class ArrayTester(unittest.TestCase):
    '''Test case for arrays.'''

    def testIntArray(self):
        intList = [1, 2, 3, 4]
        self.assertEqual(sample.sumIntArray(intList), 10)

    def testIntArrayModified(self):
        intList = [1, 2, 3, 4]
        tester = sample.ArrayModifyTest()
        self.assertEqual(tester.sumIntArray(4, intList), 10)

    def testDoubleArray(self):
        doubleList = [1.2, 2.3, 3.4, 4.5]
        self.assertEqual(sample.sumDoubleArray(doubleList), 11.4)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'certificateerrordialog.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QDialog, QDialogButtonBox,
    QLabel, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)

class Ui_CertificateErrorDialog(object):
    def setupUi(self, CertificateErrorDialog):
        if not CertificateErrorDialog.objectName():
            CertificateErrorDialog.setObjectName(u"CertificateErrorDialog")
        CertificateErrorDialog.resize(689, 204)
        self.verticalLayout = QVBoxLayout(CertificateErrorDialog)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(20, -1, 20, -1)
        self.m_iconLabel = QLabel(CertificateErrorDialog)
        self.m_iconLabel.setObjectName(u"m_iconLabel")
        self.m_iconLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.verticalLayout.addWidget(self.m_iconLabel)

        self.m_errorLabel = QLabel(CertificateErrorDialog)
        self.m_errorLabel.setObjectName(u"m_errorLabel")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.m_errorLabel.sizePolicy().hasHeightForWidth())
        self.m_errorLabel.setSizePolicy(sizePolicy)
        self.m_errorLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.m_errorLabel.setWordWrap(True)

        self.verticalLayout.addWidget(self.m_errorLabel)

        self.m_infoLabel = QLabel(CertificateErrorDialog)
        self.m_infoLabel.setObjectName(u"m_infoLabel")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.MinimumExpanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.m_infoLabel.sizePolicy().hasHeightForWidth())
        self.m_infoLabel.setSizePolicy(sizePolicy1)
        self.m_infoLabel.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)
        self.m_infoLabel.setWordWrap(True)

        self.verticalLayout.addWidget(self.m_infoLabel)

        self.verticalSpacer = QSpacerItem(20, 16, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.buttonBox = QDialogButtonBox(CertificateErrorDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.No|QDialogButtonBox.StandardButton.Yes)

        self.verticalLayout.addWidget(self.buttonBox)


        self.retranslateUi(CertificateErrorDialog)
        self.buttonBox.accepted.connect(CertificateErrorDialog.accept)
        self.buttonBox.rejected.connect(CertificateErrorDialog.reject)

        QMetaObject.connectSlotsByName(CertificateErrorDialog)
    # setupUi

    def retranslateUi(self, CertificateErrorDialog):
        CertificateErrorDialog.setWindowTitle(QCoreApplication.translate("CertificateErrorDialog", u"Dialog", None))
        self.m_iconLabel.setText(QCoreApplication.translate("CertificateErrorDialog", u"Icon", None))
        self.m_errorLabel.setText(QCoreApplication.translate("CertificateErrorDialog", u"Error", None))
        self.m_infoLabel.setText(QCoreApplication.translate("CertificateErrorDialog", u"If you wish so, you may continue with an unverified certificate. Accepting an unverified certificate mean you may not be connected with the host you tried to connect to.\n"
"\n"
"Do you wish to override the security check and continue ?   ", None))
    # retranslateUi




# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtCore import QTimer
from PySide6.QtWidgets import (QFormLayout, QHBoxLayout, QLayout, QPushButton,
                               QSpacerItem, QWidget, QWidgetItem)


class MyLayout(QLayout):
    def __init__(self, parent=None):
        QLayout.__init__(self, parent)
        self._list = []

    def addItem(self, item):
        self.add(item)

    def addWidget(self, widget):
        self.add(QWidgetItem(widget))

    def itemAt(self, index):
        if index < len(self._list):
            return self._list[index]

        return None

    def count(self):
        return len(self._list)

    def add(self, item):
        self._list.append(item)


class MissingItemAtLayout(QLayout):
    def __init__(self, parent=None):
        QLayout.__init__(self, parent)
        self._list = []

    def addItem(self, item):
        self.add(item)

    def addWidget(self, widget):
        self.add(QWidgetItem(widget))

    def count(self):
        return len(self._list)

    def add(self, item):
        self._list.append(item)

# Test if a layout implemented in python, the QWidget.setLayout works
# fine because this implement som layout functions used in glue code of
# QWidget, then in c++ when call a virtual function this need call the QLayout
# function implemented in python


class QLayoutTest(UsesQApplication):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testOwnershipTransfer(self):
        b = QPushButton("teste")
        base_ref_count = sys.getrefcount(b)
        layout = MyLayout()

        layout.addWidget(b)

        self.assertEqual(sys.getrefcount(b), base_ref_count)

        w = QWidget()

        # transfer ref
        w.setLayout(layout)

        self.assertEqual(sys.getrefcount(b), base_ref_count + 1)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testReferenceTransfer(self):
        b = QPushButton("teste")
        base_ref_count = sys.getrefcount(b)
        layout = QHBoxLayout()

        # keep ref
        layout.addWidget(b)
        self.assertEqual(sys.getrefcount(b), base_ref_count + 1)

        w = QWidget()

        # transfer ref
        w.setLayout(layout)

        self.assertEqual(sys.getrefcount(b), base_ref_count + 1)

        # release ref
        del w

        self.assertEqual(sys.getrefcount(b), base_ref_count)

    def testMissingFunctions(self):
        w = QWidget()
        b = QPushButton("test")
        layout = MissingItemAtLayout()

        layout.addWidget(b)

        self.assertRaises(RuntimeError, w.setLayout, layout)

    def testQFormLayout(self):
        w = QWidget()
        formLayout = QFormLayout()
        spacer = QSpacerItem(100, 30)
        formLayout.setItem(0, QFormLayout.ItemRole.SpanningRole, spacer)
        w.setLayout(formLayout)
        w.show()
        QTimer.singleShot(10, w.close)
        self.app.exec()
        del w
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        # PYSIDE-535: Why do I need to do it twice, here?
        gc.collect()
        self.assertRaises(RuntimeError, spacer.isEmpty)

    def testConstructorProperties(self):
        """PYSIDE-1986, test passing properties to the constructor of
           QHBoxLayout, which does not have default arguments."""
        layout = QHBoxLayout(objectName="layout", spacing=30)
        self.assertEqual(layout.spacing(), 30)
        self.assertEqual(layout.objectName(), "layout")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# Copyright (C) 2019 Andreas Beckermann
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from testbinding import PySideCPP, TestObject


class QObjectDerivedReprTest(unittest.TestCase):
    """Test the __repr__ implementation of QObject derived classes"""

    def testReprWithoutNamespace(self):
        """Test that classes outside a namespace that have a operator<<(QDebug,...) defined use that
        for __repr__"""
        t = TestObject(123)

        # We don't define __str__, so str(q) should call __repr__
        self.assertEqual(t.__repr__(), str(t))

        # __repr__ should use the operator<<(QDebug,...) implementation
        self.assertIn('TestObject(id=123)', str(t))

    def testReprWithNamespace(self):
        """Test that classes inside a namespace that have a operator<<(QDebug,...) defined use that
        for __repr__"""
        t = PySideCPP.TestObjectWithNamespace(None)

        # We don't define __str__, so str(q) should call __repr__
        self.assertEqual(t.__repr__(), str(t))

        # __repr__ should use the operator<<(QDebug,...) implementation
        self.assertIn('TestObjectWithNamespace("TestObjectWithNamespace")', str(t))

    def testReprInject(self):
        """Test that injecting __repr__ via typesystem overrides the operator<<(QDebug, ...)"""
        t = PySideCPP.TestObject2WithNamespace(None)

        # We don't define __str__, so str(q) should call __repr__
        self.assertEqual(t.__repr__(), str(t))

        # __repr__ should use the operator<<(QDebug,...) implementation
        self.assertEqual(str(t), "TestObject2WithNamespace(injected_repr)")

    def testLatin1StringField(self):
        self.assertEqual(TestObject.LATIN1_TEST_FIELD, "test")

    def testLatin1Setter(self):
        to = TestObject(123)
        value = "test"
        to.setQLatin1String(value)
        self.assertEqual(to.qLatin1String(), value)


if __name__ == '__main__':
    unittest.main()



text, ok = QInputDialog.getText(self, "QInputDialog.getText()",
                                "User name:", QLineEdit.Normal,
                                QDir.home().dirName())
if ok and text:
    textLabel.setText(text)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from textwrap import dedent

import numpy as np
from OpenGL.GL import (GL_ARRAY_BUFFER, GL_BLEND, GL_DEPTH_TEST, GL_FLOAT,
                       GL_ONE, GL_SRC_ALPHA, GL_TRIANGLE_STRIP)
from PySide6.QtCore import QSize, Slot
from PySide6.QtGui import QOpenGLFunctions
from PySide6.QtOpenGL import QOpenGLShader, QOpenGLShaderProgram
from PySide6.QtQuick import QQuickWindow, QSGRendererInterface

VERTEX_SHADER = dedent(
    """\
    attribute highp vec4 vertices;
    varying highp vec2 coords;
    void main() {
        gl_Position = vertices;
        coords = vertices.xy;
    }
    """
)
FRAGMENT_SHADER = dedent(
    """\
    uniform lowp float t;
    varying highp vec2 coords;
    void main() {
        lowp float i = 1. - (pow(abs(coords.x), 4.) + pow(abs(coords.y), 4.));
        i = smoothstep(t - 0.8, t + 0.8, i);
        i = floor(i * 20.) / 20.;
        gl_FragColor = vec4(coords * .5 + .5, i, i);
    }
    """
)


class SquircleRenderer(QOpenGLFunctions):
    def __init__(self):
        QOpenGLFunctions.__init__(self)
        self._viewport_size = QSize()
        self._t = 0.0
        self._program = None
        self._window = QQuickWindow()

    def setT(self, t):
        self._t = t

    def setViewportSize(self, size):
        self._viewport_size = size

    def setWindow(self, window):
        self._window = window

    @Slot()
    def init(self):
        if not self._program:
            rif = self._window.rendererInterface()
            assert (rif.graphicsApi() == QSGRendererInterface.GraphicsApi.OpenGL)
            self.initializeOpenGLFunctions()
            self._program = QOpenGLShaderProgram()
            self._program.addCacheableShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Vertex,
                                                           VERTEX_SHADER)
            self._program.addCacheableShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Fragment,
                                                           FRAGMENT_SHADER)
            self._program.bindAttributeLocation("vertices", 0)
            self._program.link()

    @Slot()
    def paint(self):
        # Play nice with the RHI. Not strictly needed when the scenegraph uses
        # OpenGL directly.
        self._window.beginExternalCommands()

        self._program.bind()

        self._program.enableAttributeArray(0)

        values = np.array([-1, -1, 1, -1, -1, 1, 1, 1], dtype="single")

        # This example relies on (deprecated) client-side pointers for the vertex
        # input. Therefore, we have to make sure no vertex buffer is bound.
        self.glBindBuffer(GL_ARRAY_BUFFER, 0)

        self._program.setAttributeArray(0, GL_FLOAT, values, 2)
        self._program.setUniformValue1f("t", self._t)

        self.glViewport(0, 0, self._viewport_size.width(), self._viewport_size.height())

        self.glDisable(GL_DEPTH_TEST)

        self.glEnable(GL_BLEND)
        self.glBlendFunc(GL_SRC_ALPHA, GL_ONE)

        self.glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)

        self._program.disableAttributeArray(0)
        self._program.release()

        self._window.endExternalCommands()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from functools import partial

from PySide6.QtWebEngineCore import (QWebEngineFindTextResult, QWebEnginePage)
from PySide6.QtWidgets import QLabel, QMenu, QTabBar, QTabWidget
from PySide6.QtGui import QCursor, QIcon, QKeySequence, QPixmap
from PySide6.QtCore import QUrl, Qt, Signal, Slot

from webpage import WebPage
from webview import WebView


class TabWidget(QTabWidget):
    link_hovered = Signal(str)
    load_progress = Signal(int)
    title_changed = Signal(str)
    url_changed = Signal(QUrl)
    fav_icon_changed = Signal(QIcon)
    web_action_enabled_changed = Signal(QWebEnginePage.WebAction, bool)
    dev_tools_requested = Signal(QWebEnginePage)
    find_text_finished = Signal(QWebEngineFindTextResult)

    def __init__(self, profile, parent):
        super().__init__(parent)
        self._profile = profile
        tab_bar = self.tabBar()
        tab_bar.setTabsClosable(True)
        tab_bar.setSelectionBehaviorOnRemove(QTabBar.SelectionBehavior.SelectPreviousTab)
        tab_bar.setMovable(True)
        tab_bar.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        tab_bar.customContextMenuRequested.connect(self.handle_context_menu_requested)
        tab_bar.tabCloseRequested.connect(self.close_tab)
        tab_bar.tabBarDoubleClicked.connect(self._tabbar_double_clicked)
        self.setDocumentMode(True)
        self.setElideMode(Qt.TextElideMode.ElideRight)

        self.currentChanged.connect(self.handle_current_changed)

        if profile.isOffTheRecord():
            icon = QLabel(self)
            pixmap = QPixmap(":ninja.png")
            icon.setPixmap(pixmap.scaledToHeight(tab_bar.height()))
            w = icon.pixmap().width()
            self.setStyleSheet(f"QTabWidget.tab-bar {{ left: {w}px; }}")

    @Slot(int)
    def _tabbar_double_clicked(self, index):
        if index == -1:
            self.create_tab()

    def handle_current_changed(self, index):
        if index != -1:
            view = self.web_view(index)
            if view.url():
                view.setFocus()
            self.title_changed.emit(view.title())
            self.load_progress.emit(view.load_progress())
            self.url_changed.emit(view.url())
            self.fav_icon_changed.emit(view.fav_icon())
            e = view.is_web_action_enabled(QWebEnginePage.WebAction.Back)
            self.web_action_enabled_changed.emit(QWebEnginePage.WebAction.Back, e)
            e = view.is_web_action_enabled(QWebEnginePage.WebAction.Forward)
            self.web_action_enabled_changed.emit(QWebEnginePage.WebAction.Forward, e)
            e = view.is_web_action_enabled(QWebEnginePage.WebAction.Stop)
            self.web_action_enabled_changed.emit(QWebEnginePage.WebAction.Stop, e)
            e = view.is_web_action_enabled(QWebEnginePage.WebAction.Reload)
            self.web_action_enabled_changed.emit(QWebEnginePage.WebAction.Reload, e)
        else:
            self.title_changed.emit("")
            self.load_progress.emit(0)
            self.url_changed.emit(QUrl())
            self.fav_icon_changed.emit(QIcon())
            self.web_action_enabled_changed.emit(QWebEnginePage.Back, False)
            self.web_action_enabled_changed.emit(QWebEnginePage.Forward, False)
            self.web_action_enabled_changed.emit(QWebEnginePage.Stop, False)
            self.web_action_enabled_changed.emit(QWebEnginePage.Reload, True)

    def handle_context_menu_requested(self, pos):
        menu = QMenu()
        menu.addAction("New &Tab", QKeySequence.AddTab, self.create_tab)
        index = self.tabBar().tabAt(pos)
        if index != -1:
            action = menu.addAction("Clone Tab")
            action.triggered.connect(partial(self.clone_tab, index))
            menu.addSeparator()
            action = menu.addAction("Close Tab")
            action.setShortcut(QKeySequence.Close)
            action.triggered.connect(partial(self.close_tab, index))
            action = menu.addAction("Close Other Tabs")
            action.triggered.connect(partial(self.close_other_tabs, index))
            menu.addSeparator()
            action = menu.addAction("Reload Tab")
            action.setShortcut(QKeySequence.Refresh)
            action.triggered.connect(partial(self.reload_tab, index))
        else:
            menu.addSeparator()

        menu.addAction("Reload All Tabs", self.reload_all_tabs)
        menu.exec(QCursor.pos())

    def current_web_view(self):
        return self.web_view(self.currentIndex())

    def web_view(self, index):
        return self.widget(index)

    def _title_changed(self, web_view, title):
        index = self.indexOf(web_view)
        if index != -1:
            self.setTabText(index, title)
            self.setTabToolTip(index, title)

        if self.currentIndex() == index:
            self.title_changed.emit(title)

    def _url_changed(self, web_view, url):
        index = self.indexOf(web_view)
        if index != -1:
            self.tabBar().setTabData(index, url)
        if self.currentIndex() == index:
            self.url_changed.emit(url)

    def _load_progress(self, web_view, progress):
        if self.currentIndex() == self.indexOf(web_view):
            self.load_progress.emit(progress)

    def _fav_icon_changed(self, web_view, icon):
        index = self.indexOf(web_view)
        if index != -1:
            self.setTabIcon(index, icon)
        if self.currentIndex() == index:
            self.fav_icon_changed.emit(icon)

    def _link_hovered(self, web_view, url):
        if self.currentIndex() == self.indexOf(web_view):
            self.link_hovered.emit(url)

    def _webaction_enabled_changed(self, webView, action, enabled):
        if self.currentIndex() == self.indexOf(webView):
            self.web_action_enabled_changed.emit(action, enabled)

    def _window_close_requested(self, webView):
        index = self.indexOf(webView)
        if webView.page().inspectedPage():
            self.window().close()
        elif index >= 0:
            self.close_tab(index)

    def _find_text_finished(self, webView, result):
        if self.currentIndex() == self.indexOf(webView):
            self.find_text_finished.emit(result)

    def setup_view(self, webView):
        web_page = webView.page()
        webView.titleChanged.connect(partial(self._title_changed, webView))
        webView.urlChanged.connect(partial(self._url_changed, webView))
        webView.loadProgress.connect(partial(self._load_progress, webView))
        web_page.linkHovered.connect(partial(self._link_hovered, webView))
        webView.fav_icon_changed.connect(partial(self._fav_icon_changed, webView))
        webView.web_action_enabled_changed.connect(partial(self._webaction_enabled_changed,
                                                           webView))
        web_page.windowCloseRequested.connect(partial(self._window_close_requested,
                                                      webView))
        webView.dev_tools_requested.connect(self.dev_tools_requested)
        web_page.findTextFinished.connect(partial(self._find_text_finished,
                                                  webView))

    def create_tab(self):
        web_view = self.create_background_tab()
        self.setCurrentWidget(web_view)
        return web_view

    def create_background_tab(self):
        web_view = WebView()
        web_page = WebPage(self._profile, web_view)
        web_view.set_page(web_page)
        self.setup_view(web_view)
        index = self.addTab(web_view, "(Untitled)")
        self.setTabIcon(index, web_view.fav_icon())
        # Workaround for QTBUG-61770
        web_view.resize(self.currentWidget().size())
        web_view.show()
        return web_view

    def reload_all_tabs(self):
        for i in range(0, self.count()):
            self.web_view(i).reload()

    def close_other_tabs(self, index):
        for i in range(index, self.count() - 1, -1):
            self.close_tab(i)
        for i in range(-1, index - 1, -1):
            self.close_tab(i)

    def close_tab(self, index):
        if view := self.web_view(index):
            has_focus = view.hasFocus()
            self.removeTab(index)
            if has_focus and self.count() > 0:
                self.current_web_view().setFocus()
            if self.count() == 0:
                self.create_tab()
            view.deleteLater()

    def clone_tab(self, index):
        if view := self.web_view(index):
            tab = self.create_tab()
            tab.setUrl(view.url())

    def set_url(self, url):
        if view := self.current_web_view():
            view.setUrl(url)
            view.setFocus()

    def trigger_web_page_action(self, action):
        if web_view := self.current_web_view():
            web_view.triggerPageAction(action)
            web_view.setFocus()

    def next_tab(self):
        next = self.currentIndex() + 1
        if next == self.count():
            next = 0
        self.setCurrentIndex(next)

    def previous_tab(self):
        next = self.currentIndex() - 1
        if next < 0:
            next = self.count() - 1
        self.setCurrentIndex(next)

    def reload_tab(self, index):
        if view := self.web_view(index):
            view.reload()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the opengl/textures example from Qt v6.x showing the use
   of legacy OpenGL functions with QOpenGLVersionFunctionsFactory."""

import sys
from PySide6.QtCore import QPoint, QSize, Qt, QTimer, Signal
from PySide6.QtGui import QColor, QImage, QSurfaceFormat
from PySide6.QtWidgets import QApplication, QGridLayout, QMessageBox, QWidget
from PySide6.QtOpenGL import (QOpenGLTexture, QOpenGLVersionFunctionsFactory,
                              QOpenGLVersionProfile)
from PySide6.QtOpenGLWidgets import QOpenGLWidget

try:
    from OpenGL import GL
except ImportError:
    app = QApplication(sys.argv)
    messageBox = QMessageBox(QMessageBox.Critical, "OpenGL textures",
                             "PyOpenGL must be installed to run this example.",
                             QMessageBox.Close)
    messageBox.setDetailedText("Run:\npip install PyOpenGL PyOpenGL_accelerate")
    messageBox.exec()
    sys.exit(1)

import textures_rc  # noqa: F401


class GLWidget(QOpenGLWidget):
    sharedObject = 0
    refCount = 0

    coords = (
        ((+1, -1, -1), (-1, -1, -1), (-1, +1, -1), (+1, +1, -1)),
        ((+1, +1, -1), (-1, +1, -1), (-1, +1, +1), (+1, +1, +1)),
        ((+1, -1, +1), (+1, -1, -1), (+1, +1, -1), (+1, +1, +1)),
        ((-1, -1, -1), (-1, -1, +1), (-1, +1, +1), (-1, +1, -1)),
        ((+1, -1, +1), (-1, -1, +1), (-1, -1, -1), (+1, -1, -1)),
        ((-1, -1, +1), (+1, -1, +1), (+1, +1, +1), (-1, +1, +1))
    )

    clicked = Signal()

    def __init__(self, parent):
        super().__init__(parent)

        self.clearColor = Qt.GlobalColor.black
        self.xRot = 0
        self.yRot = 0
        self.zRot = 0
        self.clearColor = QColor()
        self.lastPos = QPoint()
        self.funcs = None

    def freeGLResources(self):
        GLWidget.refCount -= 1
        if GLWidget.refCount == 0:
            self.makeCurrent()
            self.funcs.glDeleteLists(self.__class__.sharedObject, 1)

    def minimumSizeHint(self):
        return QSize(50, 50)

    def sizeHint(self):
        return QSize(200, 200)

    def rotateBy(self, xAngle, yAngle, zAngle):
        self.xRot = (self.xRot + xAngle) % 5760
        self.yRot = (self.yRot + yAngle) % 5760
        self.zRot = (self.zRot + zAngle) % 5760
        self.update()

    def setClearColor(self, color):
        self.clearColor = color
        self.update()

    def initializeGL(self):
        profile = QOpenGLVersionProfile()
        profile.setVersion(3, 2)
        profile.setProfile(QSurfaceFormat.OpenGLContextProfile.CompatibilityProfile)
        self.funcs = QOpenGLVersionFunctionsFactory.get(profile)
        self.funcs.initializeOpenGLFunctions()

        if not GLWidget.sharedObject:
            self.textures = []
            for i in range(6):
                image = QImage(f":/images/side{i + 1}.png")
                self.textures.append(QOpenGLTexture(image))
            GLWidget.sharedObject = self.makeObject()
        GLWidget.refCount += 1

        self.funcs.glEnable(GL.GL_DEPTH_TEST)
        self.funcs.glEnable(GL.GL_CULL_FACE)
        self.funcs.glEnable(GL.GL_TEXTURE_2D)

    def paintGL(self):
        self.funcs.glClearColor(self.clearColor.red(), self.clearColor.green(),
                                self.clearColor.blue(), 1)
        self.funcs.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        self.funcs.glLoadIdentity()
        self.funcs.glTranslated(0.0, 0.0, -10.0)
        self.funcs.glRotated(self.xRot / 16.0, 1.0, 0.0, 0.0)
        self.funcs.glRotated(self.yRot / 16.0, 0.0, 1.0, 0.0)
        self.funcs.glRotated(self.zRot / 16.0, 0.0, 0.0, 1.0)
        self.funcs.glCallList(GLWidget.sharedObject)

    def resizeGL(self, width, height):
        side = min(width, height)
        x = int((width - side) / 2)
        y = int((height - side) / 2)
        self.funcs.glViewport(x, y, side, side)

        self.funcs.glMatrixMode(GL.GL_PROJECTION)
        self.funcs.glLoadIdentity()
        self.funcs.glOrtho(-0.5, +0.5, +0.5, -0.5, 4.0, 15.0)
        self.funcs.glMatrixMode(GL.GL_MODELVIEW)

    def mousePressEvent(self, event):
        self.lastPos = event.position().toPoint()

    def mouseMoveEvent(self, event):
        pos = event.position().toPoint()
        dx = pos.x() - self.lastPos.x()
        dy = pos.y() - self.lastPos.y()

        if event.buttons() & Qt.MouseButton.LeftButton:
            self.rotateBy(8 * dy, 8 * dx, 0)
        elif event.buttons() & Qt.RightButton:
            self.rotateBy(8 * dy, 0, 8 * dx)

        self.lastPos = pos

    def mouseReleaseEvent(self, event):
        self.clicked.emit()

    def makeObject(self):
        dlist = self.funcs.glGenLists(1)
        self.funcs.glNewList(dlist, GL.GL_COMPILE)

        for i in range(6):
            self.textures[i].bind()

            self.funcs.glBegin(GL.GL_QUADS)
            for j in range(4):
                tx = {False: 0, True: 1}[j == 0 or j == 3]
                ty = {False: 0, True: 1}[j == 0 or j == 1]
                self.funcs.glTexCoord2d(tx, ty)
                x = 0.2 * GLWidget.coords[i][j][0]
                y = 0.2 * GLWidget.coords[i][j][1]
                z = 0.2 * GLWidget.coords[i][j][2]
                self.funcs.glVertex3d(x, y, z)

            self.funcs.glEnd()

        self.funcs.glEndList()
        return dlist


class Window(QWidget):
    NumRows = 2
    NumColumns = 3

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

        mainLayout = QGridLayout(self)
        self.glWidgets = []

        for i in range(Window.NumRows):
            self.glWidgets.append([])
            for j in range(Window.NumColumns):
                self.glWidgets[i].append(None)

        hue_div = (Window.NumRows * Window.NumColumns - 1)
        for i in range(Window.NumRows):
            for j in range(Window.NumColumns):
                clearColor = QColor()
                hue = ((i * Window.NumColumns) + j) * 255 / hue_div
                clearColor.setHsv(hue, 255, 63)

                glw = GLWidget(self)
                self.glWidgets[i][j] = glw
                glw.setClearColor(clearColor)
                glw.rotateBy(+42 * 16, +42 * 16, -21 * 16)
                mainLayout.addWidget(glw, i, j)

                glw.clicked.connect(self.setCurrentGlWidget)
                qApp.lastWindowClosed.connect(glw.freeGLResources)  # noqa: F821

        self.currentGlWidget = self.glWidgets[0][0]

        timer = QTimer(self)
        timer.timeout.connect(self.rotateOneStep)
        timer.start(20)

        self.setWindowTitle(self.tr("Textures"))

    def setCurrentGlWidget(self):
        self.currentGlWidget = self.sender()

    def rotateOneStep(self):
        if self.currentGlWidget:
            self.currentGlWidget.rotateBy(+2 * 16, +2 * 16, -1 * 16)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = Window()
    window.show()
    sys.exit(app.exec())



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QLineEdit, QSizePolicy, QWidget, QVBoxLayout
from PySide6.QtGui import QAction
from PySide6.QtCore import QUrl, Qt, Slot

from webpage import WebPage


class WebPopupWindow(QWidget):

    def __init__(self, view, profile, parent=None):
        super().__init__(parent, Qt.Window)
        self._url_line_edit = QLineEdit()
        self._fav_action = QAction(self)
        self._view = view

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)

        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._url_line_edit)
        layout.addWidget(self._view)

        self._view.setPage(WebPage(profile, self._view))
        self._view.setFocus()

        self._url_line_edit.setReadOnly(True)
        self._url_line_edit.addAction(self._fav_action, QLineEdit.ActionPosition.LeadingPosition)

        self._view.titleChanged.connect(self.setWindowTitle)
        self._view.urlChanged.connect(self._url_changed)
        self._view.fav_icon_changed.connect(self._fav_action.setIcon)
        p = self._view.page()
        p.geometryChangeRequested.connect(self.handle_geometry_change_requested)
        p.windowCloseRequested.connect(self.close)

    @Slot(QUrl)
    def _url_changed(self, url):
        self._url_line_edit.setText(url.toDisplayString())

    def view(self):
        return self._view

    def handle_geometry_change_requested(self, newGeometry):
        if window := self.windowHandle():
            self.setGeometry(newGeometry.marginsRemoved(window.frameMargins()))
        self.show()
        self._view.setFocus()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import Enum, auto

from PySide6.QtMultimediaWidgets import QVideoWidget
from PySide6.QtMultimedia import (QCapturableWindow, QMediaCaptureSession,
                                  QScreenCapture, QWindowCapture)
from PySide6.QtWidgets import (QGridLayout, QLabel, QListView,
                               QMessageBox, QPushButton, QWidget)
from PySide6.QtGui import QAction, QGuiApplication
from PySide6.QtCore import QItemSelection, Qt, Slot

from screenlistmodel import ScreenListModel
from windowlistmodel import WindowListModel


class SourceType(Enum):
    Screen = auto()
    Window = auto()


class ScreenCapturePreview(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._source = SourceType.Screen

        self._screen_capture = QScreenCapture(self)
        self._media_capture_session = QMediaCaptureSession(self)
        self._video_widget = QVideoWidget(self)
        self._screen_list_view = QListView(self)
        self._screen_label = QLabel("Select screen to capture:", self)
        self._video_widget_label = QLabel("Capture output:", self)
        self._start_stop_button = QPushButton(self)
        self._status_label = QLabel(self)

        self._screen_list_model = ScreenListModel(self)

        # Setup QScreenCapture with initial source:
        self.setScreen(QGuiApplication.primaryScreen())
        self._screen_capture.start()
        self._media_capture_session.setScreenCapture(self._screen_capture)
        self._media_capture_session.setVideoOutput(self._video_widget)

        self._screen_list_view.setModel(self._screen_list_model)

        self._window_list_view = QListView(self)
        self._window_capture = QWindowCapture(self)
        self._media_capture_session.setWindowCapture(self._window_capture)
        self._window_label = QLabel("Select window to capture:", self)

        self._window_list_model = WindowListModel(self)
        self._window_list_view.setModel(self._window_list_model)
        update_action = QAction("Update windows List", self)
        update_action.triggered.connect(self._window_list_model.populate)
        self._window_list_view.addAction(update_action)
        self._window_list_view.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)

        grid_layout = QGridLayout(self)
        grid_layout.addWidget(self._screen_label, 0, 0)
        grid_layout.addWidget(self._screen_list_view, 1, 0)
        grid_layout.addWidget(self._start_stop_button, 4, 0)
        grid_layout.addWidget(self._video_widget_label, 0, 1)
        grid_layout.addWidget(self._video_widget, 1, 1, 4, 1)
        grid_layout.addWidget(self._window_label, 2, 0)
        grid_layout.addWidget(self._window_list_view, 3, 0)
        grid_layout.addWidget(self._status_label, 5, 0, 1, 2)

        grid_layout.setColumnStretch(1, 1)
        grid_layout.setRowStretch(1, 1)
        grid_layout.setColumnMinimumWidth(0, 400)
        grid_layout.setColumnMinimumWidth(1, 400)
        grid_layout.setRowMinimumHeight(3, 1)

        selection_model = self._screen_list_view.selectionModel()
        selection_model.selectionChanged.connect(self.on_current_screen_selection_changed)
        selection_model = self._window_list_view.selectionModel()
        selection_model.selectionChanged.connect(self.on_current_window_selection_changed)

        self._start_stop_button.clicked.connect(self.on_start_stop_button_clicked)
        self._screen_capture.errorOccurred.connect(self.on_screen_capture_error_occured,
                                                   Qt.ConnectionType.QueuedConnection)
        self._window_capture.errorOccurred.connect(self.on_window_capture_error_occured,
                                                   Qt.ConnectionType.QueuedConnection)
        self.update_active(SourceType.Screen, True)

    @Slot(QItemSelection)
    def on_current_screen_selection_changed(self, selection):
        self.clear_error_string()
        if indexes := selection.indexes():
            self._screen_capture.setScreen(self._screen_list_model.screen(indexes[0]))
            self.update_active(SourceType.Screen, self.is_active())
            self._window_list_view.clearSelection()
        else:
            self._screen_capture.setScreen(None)

    @Slot(QItemSelection)
    def on_current_window_selection_changed(self, selection):
        self.clear_error_string()
        if indexes := selection.indexes():
            window = self._window_list_model.window(indexes[0])
            if not window.isValid():
                m = "The window is no longer valid. Update the list of windows?"
                answer = QMessageBox.question(self, "Invalid window", m)
                if answer == QMessageBox.Yes:
                    self.update_active(SourceType.Window, False)
                    self._window_list_view.clearSelection()
                    self._window_list_model.populate()
                    return
            self._window_capture.setWindow(window)
            self.update_active(SourceType.Window, self.is_active())
            self._screen_list_view.clearSelection()
        else:
            self._window_capture.setWindow(QCapturableWindow())

    @Slot(QWindowCapture.Error, str)
    def on_window_capture_error_occured(self, error, error_string):
        self.set_error_string("QWindowCapture: Error occurred " + error_string)

    @Slot(QScreenCapture.Error, str)
    def on_screen_capture_error_occured(self, error, error_string):
        self.set_error_string("QScreenCapture: Error occurred " + error_string)

    def set_error_string(self, t):
        self._status_label.setStyleSheet("background-color: rgb(255, 0, 0);")
        self._status_label.setText(t)

    def clear_error_string(self):
        self._status_label.clear()
        self._status_label.setStyleSheet("")

    @Slot()
    def on_start_stop_button_clicked(self):
        self.clear_error_string()
        self.update_active(self._source_type, not self.is_active())

    def update_start_stop_button_text(self):
        active = self.is_active()
        if self._source_type == SourceType.Window:
            m = "Stop window capture" if active else "Start window capture"
            self._start_stop_button.setText(m)
        elif self._source_type == SourceType.Screen:
            m = "Stop screen capture" if active else "Start screen capture"
            self._start_stop_button.setText(m)

    def update_active(self, source_type, active):
        self._source_type = source_type
        self._screen_capture.setActive(active and source_type == SourceType.Screen)
        self._window_capture.setActive(active and source_type == SourceType.Window)

        self.update_start_stop_button_text()

    def is_active(self):
        if self._source_type == SourceType.Window:
            return self._window_capture.isActive()
        if self._source_type == SourceType.Screen:
            return self._screen_capture.isActive()
        return False



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtCore import QTimer
from PySide6.QtGui import QPicture, QPainter
from PySide6.QtWidgets import QWidget


class MyWidget(QWidget):
    def __init__(self, picture):
        super().__init__()
        self._picture = picture

    def paintEvent(self, e):
        with QPainter(self) as p:
            p.drawPicture(0, 0, self._picture)
        QTimer.singleShot(0, qApp.quit)  # noqa: F821


class QPictureTest(UsesQApplication):
    def testFromData(self):
        picture = QPicture()
        with QPainter(picture) as painter:
            painter.drawEllipse(10, 20, 80, 70)

        data = picture.data()
        picture2 = QPicture()
        picture2.setData(data)

        self.assertEqual(picture2.data(), picture.data())

        w = MyWidget(picture2)

        w.show()
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/mainwindows/mdi example from Qt v5.x, originating from PyQt"""

from argparse import ArgumentParser, RawTextHelpFormatter
from functools import partial
import sys

from PySide6.QtCore import (QByteArray, QFile, QFileInfo, QSettings,
                            QSaveFile, QTextStream, Qt, Slot)
from PySide6.QtGui import QAction, QIcon, QKeySequence
from PySide6.QtWidgets import (QApplication, QFileDialog, QMainWindow,
                               QMdiArea, QMessageBox, QTextEdit)


class MdiChild(QTextEdit):
    sequence_number = 1

    def __init__(self):
        super().__init__()

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self._is_untitled = True

    def new_file(self):
        self._is_untitled = True
        self._cur_file = f"document{MdiChild.sequence_number}.txt"
        MdiChild.sequence_number += 1
        self.setWindowTitle(f"{self._cur_file}[*]")

        self.document().contentsChanged.connect(self.document_was_modified)

    def load_file(self, fileName):
        file = QFile(fileName)
        if not file.open(QFile.OpenModeFlag.ReadOnly | QFile.OpenModeFlag.Text):
            reason = file.errorString()
            message = f"Cannot read file {fileName}:\n{reason}."
            QMessageBox.warning(self, "MDI", message)
            return False

        instr = QTextStream(file)
        with QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor):
            self.setPlainText(instr.readAll())

        self.set_current_file(fileName)

        self.document().contentsChanged.connect(self.document_was_modified)

        return True

    def save(self):
        if self._is_untitled:
            return self.save_as()
        else:
            return self.save_file(self._cur_file)

    def save_as(self):
        fileName, _ = QFileDialog.getSaveFileName(self, "Save As", self._cur_file)
        if not fileName:
            return False

        return self.save_file(fileName)

    def save_file(self, fileName):
        error = None
        with QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor):
            file = QSaveFile(fileName)
            if file.open(QFile.OpenModeFlag.WriteOnly | QFile.OpenModeFlag.Text):
                outstr = QTextStream(file)
                outstr << self.toPlainText()
                if not file.commit():
                    reason = file.errorString()
                    error = f"Cannot write file {fileName}:\n{reason}."
            else:
                reason = file.errorString()
                error = f"Cannot open file {fileName}:\n{reason}."

        if error:
            QMessageBox.warning(self, "MDI", error)
            return False

        self.set_current_file(fileName)
        return True

    def user_friendly_current_file(self):
        return self.stripped_name(self._cur_file)

    def current_file(self):
        return self._cur_file

    def closeEvent(self, event):
        if self.maybe_save():
            event.accept()
        else:
            event.ignore()

    def document_was_modified(self):
        self.setWindowModified(self.document().isModified())

    def maybe_save(self):
        if self.document().isModified():
            f = self.user_friendly_current_file()
            message = f"'{f}' has been modified.\nDo you want to save your changes?"
            ret = QMessageBox.warning(self, "MDI", message,
                                      QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)

            if ret == QMessageBox.Save:
                return self.save()

            if ret == QMessageBox.Cancel:
                return False

        return True

    def set_current_file(self, fileName):
        self._cur_file = QFileInfo(fileName).canonicalFilePath()
        self._is_untitled = False
        self.document().setModified(False)
        self.setWindowModified(False)
        self.setWindowTitle(f"{self.user_friendly_current_file()}[*]")

    def stripped_name(self, fullFileName):
        return QFileInfo(fullFileName).fileName()


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self._mdi_area = QMdiArea()
        self._mdi_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self._mdi_area.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.setCentralWidget(self._mdi_area)

        self._mdi_area.subWindowActivated.connect(self.update_menus)

        self.create_actions()
        self.create_menus()
        self.create_tool_bars()
        self.create_status_bar()
        self.update_menus()

        self.read_settings()

        self.setWindowTitle("MDI")

    def closeEvent(self, event):
        self._mdi_area.closeAllSubWindows()
        if self._mdi_area.currentSubWindow():
            event.ignore()
        else:
            self.write_settings()
            event.accept()

    @Slot()
    def new_file(self):
        child = self.create_mdi_child()
        child.new_file()
        child.show()

    @Slot()
    def open(self):
        file_name, _ = QFileDialog.getOpenFileName(self)
        if file_name:
            if existing := self.find_mdi_child(file_name):
                self._mdi_area.setActiveSubWindow(existing)
            else:
                self.load(file_name)

    def load(self, file_name):
        child = self.create_mdi_child()
        if child.load_file(file_name):
            self.statusBar().showMessage("File loaded", 2000)
            child.show()
        else:
            child.close()

    @Slot()
    def save(self):
        if self.active_mdi_child() and self.active_mdi_child().save():
            self.statusBar().showMessage("File saved", 2000)

    @Slot()
    def save_as(self):
        if self.active_mdi_child() and self.active_mdi_child().save_as():
            self.statusBar().showMessage("File saved", 2000)

    @Slot()
    def cut(self):
        if self.active_mdi_child():
            self.active_mdi_child().cut()

    @Slot()
    def copy(self):
        if self.active_mdi_child():
            self.active_mdi_child().copy()

    @Slot()
    def paste(self):
        if self.active_mdi_child():
            self.active_mdi_child().paste()

    @Slot()
    def about(self):
        QMessageBox.about(self, "About MDI",
                          "The <b>MDI</b> example demonstrates how to write multiple "
                          "document interface applications using Qt.")

    @Slot()
    def update_menus(self):
        has_mdi_child = (self.active_mdi_child() is not None)
        self._save_act.setEnabled(has_mdi_child)
        self._save_as_act.setEnabled(has_mdi_child)
        self._paste_act.setEnabled(has_mdi_child)
        self._close_act.setEnabled(has_mdi_child)
        self._close_all_act.setEnabled(has_mdi_child)
        self._tile_act.setEnabled(has_mdi_child)
        self._cascade_act.setEnabled(has_mdi_child)
        self._next_act.setEnabled(has_mdi_child)
        self._previous_act.setEnabled(has_mdi_child)
        self._separator_act.setVisible(has_mdi_child)

        has_selection = (self.active_mdi_child() is not None
                         and self.active_mdi_child().textCursor().hasSelection())
        self._cut_act.setEnabled(has_selection)
        self._copy_act.setEnabled(has_selection)

    @Slot()
    def update_window_menu(self):
        self._window_menu.clear()
        self._window_menu.addAction(self._close_act)
        self._window_menu.addAction(self._close_all_act)
        self._window_menu.addSeparator()
        self._window_menu.addAction(self._tile_act)
        self._window_menu.addAction(self._cascade_act)
        self._window_menu.addSeparator()
        self._window_menu.addAction(self._next_act)
        self._window_menu.addAction(self._previous_act)
        self._window_menu.addAction(self._separator_act)

        windows = self._mdi_area.subWindowList()
        self._separator_act.setVisible(len(windows) != 0)

        for i, window in enumerate(windows):
            child = window.widget()

            f = child.user_friendly_current_file()
            text = f'{i + 1} {f}'
            if i < 9:
                text = '&' + text

            action = self._window_menu.addAction(text)
            action.setCheckable(True)
            action.setChecked(child is self.active_mdi_child())
            slot_func = partial(self.set_active_sub_window, window=window)
            action.triggered.connect(slot_func)

    def create_mdi_child(self):
        child = MdiChild()
        self._mdi_area.addSubWindow(child)

        child.copyAvailable.connect(self._cut_act.setEnabled)
        child.copyAvailable.connect(self._copy_act.setEnabled)

        return child

    def create_actions(self):

        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentNew)
        self._new_act = QAction(icon, "&New", self, shortcut=QKeySequence.StandardKey.New,
                                statusTip="Create a new file", triggered=self.new_file)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentOpen)
        self._open_act = QAction(icon, "&Open...", self, shortcut=QKeySequence.StandardKey.Open,
                                 statusTip="Open an existing file", triggered=self.open)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentSave)
        self._save_act = QAction(icon, "&Save", self,
                                 shortcut=QKeySequence.StandardKey.Save,
                                 statusTip="Save the document to disk", triggered=self.save)

        self._save_as_act = QAction("Save &As...", self,
                                    shortcut=QKeySequence.StandardKey.SaveAs,
                                    statusTip="Save the document under a new name",
                                    triggered=self.save_as)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.ApplicationExit)
        self._exit_act = QAction(icon, "E&xit", self, shortcut=QKeySequence.StandardKey.Quit,
                                 statusTip="Exit the application",
                                 triggered=QApplication.instance().closeAllWindows)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditCut)
        self._cut_act = QAction(icon, "Cu&t", self,
                                shortcut=QKeySequence.StandardKey.Cut,
                                statusTip="Cut the current selection's contents to the clipboard",
                                triggered=self.cut)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditCopy)
        self._copy_act = QAction(icon, "&Copy", self,
                                 shortcut=QKeySequence.StandardKey.Copy,
                                 statusTip="Copy the current selection's contents to the clipboard",
                                 triggered=self.copy)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditPaste)
        self._paste_act = QAction(icon, "&Paste", self,
                                  shortcut=QKeySequence.StandardKey.Paste,
                                  statusTip="Paste the clipboard's contents into the current "
                                            "selection",
                                  triggered=self.paste)

        self._close_act = QAction("Cl&ose", self,
                                  statusTip="Close the active window",
                                  triggered=self._mdi_area.closeActiveSubWindow)

        self._close_all_act = QAction("Close &All", self,
                                      statusTip="Close all the windows",
                                      triggered=self._mdi_area.closeAllSubWindows)

        self._tile_act = QAction("&Tile", self, statusTip="Tile the windows",
                                 triggered=self._mdi_area.tileSubWindows)

        self._cascade_act = QAction("&Cascade", self,
                                    statusTip="Cascade the windows",
                                    triggered=self._mdi_area.cascadeSubWindows)

        self._next_act = QAction("Ne&xt", self, shortcut=QKeySequence.StandardKey.NextChild,
                                 statusTip="Move the focus to the next window",
                                 triggered=self._mdi_area.activateNextSubWindow)

        self._previous_act = QAction("Pre&vious", self,
                                     shortcut=QKeySequence.StandardKey.PreviousChild,
                                     statusTip="Move the focus to the previous window",
                                     triggered=self._mdi_area.activatePreviousSubWindow)

        self._separator_act = QAction(self)
        self._separator_act.setSeparator(True)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.HelpAbout)
        self._about_act = QAction(icon, "&About", self,
                                  statusTip="Show the application's About box",
                                  triggered=self.about)

        self._about_qt_act = QAction("About &Qt", self,
                                     statusTip="Show the Qt library's About box",
                                     triggered=QApplication.instance().aboutQt)

    def create_menus(self):
        self._file_menu = self.menuBar().addMenu("&File")
        self._file_menu.addAction(self._new_act)
        self._file_menu.addAction(self._open_act)
        self._file_menu.addAction(self._save_act)
        self._file_menu.addAction(self._save_as_act)
        self._file_menu.addSeparator()
        action = self._file_menu.addAction("Switch layout direction")
        action.triggered.connect(self.switch_layout_direction)
        self._file_menu.addAction(self._exit_act)

        self._edit_menu = self.menuBar().addMenu("&Edit")
        self._edit_menu.addAction(self._cut_act)
        self._edit_menu.addAction(self._copy_act)
        self._edit_menu.addAction(self._paste_act)

        self._window_menu = self.menuBar().addMenu("&Window")
        self.update_window_menu()
        self._window_menu.aboutToShow.connect(self.update_window_menu)

        self.menuBar().addSeparator()

        self._help_menu = self.menuBar().addMenu("&Help")
        self._help_menu.addAction(self._about_act)
        self._help_menu.addAction(self._about_qt_act)

    def create_tool_bars(self):
        self._file_tool_bar = self.addToolBar("File")
        self._file_tool_bar.addAction(self._new_act)
        self._file_tool_bar.addAction(self._open_act)
        self._file_tool_bar.addAction(self._save_act)

        self._edit_tool_bar = self.addToolBar("Edit")
        self._edit_tool_bar.addAction(self._cut_act)
        self._edit_tool_bar.addAction(self._copy_act)
        self._edit_tool_bar.addAction(self._paste_act)

    def create_status_bar(self):
        self.statusBar().showMessage("Ready")

    def read_settings(self):
        settings = QSettings('QtProject', 'MDI Example')
        geometry = settings.value('geometry', QByteArray())
        if geometry.size():
            self.restoreGeometry(geometry)

    def write_settings(self):
        settings = QSettings('QtProject', 'MDI Example')
        settings.setValue('geometry', self.saveGeometry())

    def active_mdi_child(self):
        if active_sub_window := self._mdi_area.activeSubWindow():
            return active_sub_window.widget()
        return None

    def find_mdi_child(self, fileName):
        canonical_file_path = QFileInfo(fileName).canonicalFilePath()

        for window in self._mdi_area.subWindowList():
            if window.widget().current_file() == canonical_file_path:
                return window
        return None

    @Slot()
    def switch_layout_direction(self):
        if self.layoutDirection() == Qt.LeftToRight:
            QApplication.setLayoutDirection(Qt.RightToLeft)
        else:
            QApplication.setLayoutDirection(Qt.LeftToRight)

    def set_active_sub_window(self, window):
        if window:
            self._mdi_area.setActiveSubWindow(window)


if __name__ == '__main__':
    argument_parser = ArgumentParser(description='MDI Example',
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("files", help="Files",
                                 nargs='*', type=str)
    options = argument_parser.parse_args()

    app = QApplication(sys.argv)

    main_win = MainWindow()
    for f in options.files:
        main_win.load(f)
    main_win.show()
    sys.exit(app.exec())
